@@ -23,7 +23,8 @@
  *        Authored By: Sam Spilsbury <sam.spilsbury@canonical.com>
  */
 
-#include "gtk-window-decorator.h" 
+#include "gtk-window-decorator.h"
+#include "gwd-settings-interface.h"
 
 void
 move_resize_window (WnckWindow *win,
@@ -44,8 +45,6 @@ move_resize_window (WnckWindow *win,
     if (action_menu_mapped)
     {
 	gtk_object_destroy (GTK_OBJECT (action_menu));
-	action_menu_mapped = FALSE;
-	action_menu = NULL;
 	return;
     }
 
@@ -133,6 +132,11 @@ close_button_event (WnckWindow *win,
     decor_t *d = g_object_get_data (G_OBJECT (win), "decor");
     guint   state = d->button_states[BUTTON_CLOSE];
 
+    if (action_menu_mapped && gtkwd_type == GButtonPress)
+    {
+	gtk_object_destroy (GTK_OBJECT (action_menu));
+    }
+
     common_button_event (win, gtkwd_event, gtkwd_type,
 			 BUTTON_CLOSE, 1, _("Close Window"));
 
@@ -155,6 +159,11 @@ max_button_event (WnckWindow *win,
     decor_t *d = g_object_get_data (G_OBJECT (win), "decor");
     guint   state = d->button_states[BUTTON_MAX];
 
+    if (action_menu_mapped && gtkwd_type == GButtonPress)
+    {
+	gtk_object_destroy (GTK_OBJECT (action_menu));
+    }
+
     if (wnck_window_is_maximized (win))
 	common_button_event (win, gtkwd_event, gtkwd_type, BUTTON_MAX,
 			     3, _("Unmaximize Window"));
@@ -209,6 +218,11 @@ min_button_event (WnckWindow *win,
     decor_t *d = g_object_get_data (G_OBJECT (win), "decor");
     guint   state = d->button_states[BUTTON_MIN];
 
+    if (action_menu_mapped && gtkwd_type == GButtonPress)
+    {
+	gtk_object_destroy (GTK_OBJECT (action_menu));
+    }
+
     common_button_event (win, gtkwd_event, gtkwd_type,
 			 BUTTON_MIN, 1, _("Minimize Window"));
 
@@ -685,7 +699,7 @@ find_event_callback_for_point (decor_t *d,
     int    i, j;
     BoxPtr box;
 
-    for (i = 0; i < BUTTON_NUM; i++)
+    for (i = 0; i < BUTTON_NUM; ++i)
     {
 	box = &d->button_windows[i].pos;
 	if (x >= box->x1 && x <= box->x2 &&
@@ -704,9 +718,9 @@ find_event_callback_for_point (decor_t *d,
 	}
     }
 
-    for (i = 0; i < 3; i++)
+    for (i = 0; i < 3; ++i)
     {
-	for (j = 0; j < 3; j++)
+	for (j = 0; j < 3; ++j)
 	{
 	    box = &d->event_windows[i][j].pos;
 	    if (x >= box->x1 && x <= box->x2 &&
@@ -734,15 +748,15 @@ find_leave_event_callback (decor_t *d)
 {
     int i, j;
 
-    for (i = 0; i < BUTTON_NUM; i++)
+    for (i = 0; i < BUTTON_NUM; ++i)
     {
 	if (d->last_pos_entered == &d->button_windows[i].pos)
 	    return d->button_windows[i].callback;
     }
 
-    for (i = 0; i < 3; i++)
+    for (i = 0; i < 3; ++i)
     {
-	for (j = 0; j < 3; j++)
+	for (j = 0; j < 3; ++j)
 	{
 	    if (d->last_pos_entered == &d->event_windows[i][j].pos)
 		return d->event_windows[i][j].callback;
@@ -990,9 +1004,9 @@ event_filter_func (GdkXEvent *gdkxevent,
 	    GdkScreen  *g_screen = gdk_display_get_default_screen (gdkdisplay);
 	    Window     root = GDK_WINDOW_XWINDOW (gdk_screen_get_root_window (g_screen));
 	    WnckScreen *screen;
-	    
+
 	    screen = wnck_screen_get_for_root (root);
-	    
+
 	    if (screen)
 	    {
 		if (gwd_process_decor_shadow_property_update ())
@@ -1092,17 +1106,6 @@ event_filter_func (GdkXEvent *gdkxevent,
 	    if (win)
 		update_window_decoration_size (win);
 	}
-	else if (xevent->xclient.message_type == decor_delete_pixmap_atom)
-	{
-	    gconstpointer key = GINT_TO_POINTER (xevent->xclient.data.l[0]);
-	    decor_t *d = g_hash_table_lookup (destroyed_pixmaps_table, key);
-
-	    if (d != NULL)
-	    {
-		g_hash_table_remove (d->old_pixmaps, key);
-		g_hash_table_remove (destroyed_pixmaps_table, key);
-	    }
-	}
     default:
 	break;
     }
@@ -1122,14 +1125,14 @@ event_filter_func (GdkXEvent *gdkxevent,
 		event_callback   cb = NULL;
 		Window           w = xevent->xany.window;
 
-		for (i = 0; i < 3; i++)
-		    for (j = 0; j < 3; j++)
+		for (i = 0; i < 3; ++i)
+		    for (j = 0; j < 3; ++j)
 			if (d->event_windows[i][j].window == w)
 			    cb = d->event_windows[i][j].callback;
 
 		if (!cb)
 		{
-		    for (i = 0; i < BUTTON_NUM; i++)
+		    for (i = 0; i < BUTTON_NUM; ++i)
 			if (d->button_windows[i].window == w)
 			    cb = d->button_windows[i].callback;
 		}
