@@ -21,9 +21,8 @@
 #define _COMPIZ_GWD_SETTINGS_NOTIFIED_H
 
 #include <glib-object.h>
-
-typedef struct _GWDSettingsNotified GWDSettingsNotified;
-typedef struct _WnckScreen	    WnckScreen;
+#include <libwnck/screen.h>
+#include <gwd-fwd.h>
 
 G_BEGIN_DECLS
 
