@@ -255,8 +255,6 @@ restack_window (WnckWindow *win,
     if (action_menu_mapped)
     {
 	gtk_object_destroy (GTK_OBJECT (action_menu));
-	action_menu_mapped = FALSE;
-	action_menu = NULL;
 	return;
     }
 
@@ -376,9 +374,9 @@ add_frame_window (WnckWindow *win,
     {
 	d->frame_window = NULL;
 
-	for (i = 0; i < 3; i++)
+	for (i = 0; i < 3; ++i)
 	{
-	    for (j = 0; j < 3; j++)
+	    for (j = 0; j < 3; ++j)
 	    {
 		d->event_windows[i][j].window =
 		XCreateWindow (xdisplay,
@@ -395,7 +393,7 @@ add_frame_window (WnckWindow *win,
 
 	attr.event_mask |= ButtonReleaseMask;
 
-	for (i = 0; i < BUTTON_NUM; i++)
+	for (i = 0; i < BUTTON_NUM; ++i)
 	{
 	    d->button_windows[i].window =
 	    XCreateWindow (xdisplay,
@@ -414,8 +412,8 @@ add_frame_window (WnckWindow *win,
 	if (get_mwm_prop (xid) & (MWM_DECOR_ALL | MWM_DECOR_TITLE))
 	    d->decorated = TRUE;
 
-	for (i = 0; i < 3; i++)
-	    for (j = 0; j < 3; j++)
+	for (i = 0; i < 3; ++i)
+	    for (j = 0; j < 3; ++j)
 	    {
 		Window win = d->event_windows[i][j].window;
 		g_hash_table_insert (frame_table,
@@ -423,7 +421,7 @@ add_frame_window (WnckWindow *win,
 				     GINT_TO_POINTER (xid));
 	    }
 
-	for (i = 0; i < BUTTON_NUM; i++)
+	for (i = 0; i < BUTTON_NUM; ++i)
 	    g_hash_table_insert (frame_table,
 				 GINT_TO_POINTER (d->button_windows[i].window),
 				 GINT_TO_POINTER (xid));
@@ -447,8 +445,8 @@ add_frame_window (WnckWindow *win,
     }
     else
     {
-	for (i = 0; i < 3; i++)
-	    for (j = 0; j < 3; j++)
+	for (i = 0; i < 3; ++i)
+	    for (j = 0; j < 3; ++j)
 		d->event_windows[i][j].window = None;
     }
 
@@ -467,16 +465,16 @@ remove_frame_window (WnckWindow *win)
     {
 	int i, j;
 
-	for (i = 0; i < 3; i++)
+	for (i = 0; i < 3; ++i)
 	{
-	    for (j = 0; j < 3; j++)
+	    for (j = 0; j < 3; ++j)
 	    {
 		XDestroyWindow (xdisplay, d->event_windows[i][j].window);
 		d->event_windows[i][j].window = None;
 	    }
 	}
 
-	for (i = 0; i < BUTTON_NUM; i++)
+	for (i = 0; i < BUTTON_NUM; ++i)
 	{
 	    XDestroyWindow (xdisplay, d->button_windows[i].window);
 	    d->button_windows[i].window = None;
@@ -497,12 +495,6 @@ remove_frame_window (WnckWindow *win)
 	d->buffer_pixmap = NULL;
     }
 
-    if (d->old_pixmaps)
-    {
-	g_hash_table_destroy (d->old_pixmaps);
-	d->old_pixmaps = NULL;
-    }
-
     if (d->cr)
     {
 	cairo_destroy (d->cr);
@@ -805,11 +797,11 @@ window_opened (WnckScreen *screen,
     if (!d)
 	return;
 
-    for (i = 0; i < 3; i++)
-	for (j = 0; j < 3; j++)
+    for (i = 0; i < 3; ++i)
+	for (j = 0; j < 3; ++j)
 	    d->event_windows[i][j].callback = callback[i][j];
 
-    for (i = 0; i < BUTTON_NUM; i++)
+    for (i = 0; i < BUTTON_NUM; ++i)
 	d->button_windows[i].callback = button_callback[i];
 
     wnck_window_get_client_window_geometry (win, NULL, NULL,
