@@ -27,15 +27,7 @@
 
 COMPIZ_PLUGIN_20090315 (cubeaddon, CubeaddonPluginVTable);
 
-const unsigned short CUBEADDON_GRID_SIZE = 100;
-const unsigned short CAP_ELEMENTS    = 15;
-const unsigned int   CAP_NVERTEX	    = (((CAP_ELEMENTS * (CAP_ELEMENTS + 1)) + 2) * 3);
-const unsigned int   CAP_NIDX	    = (CAP_ELEMENTS * (CAP_ELEMENTS - 1) * 4);
-
-const unsigned int   CAP_NIMGVERTEX  = (((CAP_ELEMENTS + 1) * (CAP_ELEMENTS + 1)) * 5);
-const unsigned int   CAP_NIMGIDX	    = (CAP_ELEMENTS * CAP_ELEMENTS * 4);
-
-const float RAD2I1024 = 162.9746617f;
+unsigned short COLORMAX = 0xffff;
 
 /*
  * Initiate a CubeCap
@@ -52,6 +44,16 @@ CubeaddonScreen::CubeCap::CubeCap ()
 void
 CubeaddonScreen::CubeCap::load (bool scale, bool aspect, bool clamp)
 {
+    /* we need to clear the texture, if no texture files are specified */
+    if (mFiles.empty ())
+    {
+	mTexture.clear ();
+	mLoaded = false;
+	mCurrent = 0;
+
+	return;
+    }
+
     CompSize tSize;
     float    xScale, yScale;
 
@@ -61,12 +63,6 @@ CubeaddonScreen::CubeCap::load (bool scale, bool aspect, bool clamp)
 
     mLoaded = false;
 
-    if (mFiles.empty ())
-	return;
-
-    mCurrent = mCurrent % mFiles.size ();
-
-
     CompString imgName = mFiles[mCurrent].s ();
     CompString pname = "cubeaddon";
     mTexture = GLTexture::readImageToTexture (imgName, pname, tSize);
@@ -118,10 +114,13 @@ CubeaddonScreen::CubeCap::load (bool scale, bool aspect, bool clamp)
     {
 	if (GL::textureBorderClamp)
 	{
+#ifndef USE_GLES
+	    /* FIXME: Simulate with shaders */
 	    glTexParameteri (mTexture[0]->target (), GL_TEXTURE_WRAP_S,
 			     GL_CLAMP_TO_BORDER);
 	    glTexParameteri (mTexture[0]->target (), GL_TEXTURE_WRAP_T,
 			     GL_CLAMP_TO_BORDER);
+#endif
 	}
 	else
 	{
@@ -148,23 +147,25 @@ bool
 CubeaddonScreen::changeCap (bool top, int change)
 {
     CubeCap *cap = (top)? &mTopCap : &mBottomCap;
-    if (cap->mFiles.size ())
-    {
-	int count = cap->mFiles.size ();
+
+    int count = cap->mFiles.size ();
+
+    /* mCurrent just changes in this case */
+    if (count && change)
 	cap->mCurrent = (cap->mCurrent + change + count) % count;
-	if (top)
-	{
-	    cap->load (optionGetTopScale (), optionGetTopAspect (),
-		       optionGetTopClamp ());
-	}
-	else
-	{
-	    cap->load (optionGetBottomScale (), optionGetBottomAspect (),
-		       optionGetBottomClamp ());
-	    cap->mTexMat.scale (1.0, -1.0, 1.0);
-	}
-	cScreen->damageScreen ();
+
+    if (top)
+    {
+	cap->load (optionGetTopScale (), optionGetTopAspect (),
+		   optionGetTopClamp ());
     }
+    else
+    {
+	cap->load (optionGetBottomScale (), optionGetBottomAspect (),
+		   optionGetBottomClamp ());
+	cap->mTexMat.scale (1.0, -1.0, 1.0);
+    }
+    cScreen->damageScreen ();
 
     return false;
 }
@@ -172,7 +173,6 @@ CubeaddonScreen::changeCap (bool top, int change)
 bool
 CubeaddonScreen::setOption (const CompString &name, CompOption::Value &value)
 {
-
     unsigned int index;
 
     bool rv = CubeaddonOptions::setOption (name, value);
@@ -211,44 +211,62 @@ CubeaddonScreen::setOption (const CompString &name, CompOption::Value &value)
 void
 CubeaddonScreen::drawBasicGround ()
 {
-    float i;
-
-    glPushMatrix ();
-
     glEnable (GL_BLEND);
     glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+    float i = optionGetIntensity () * 2;
 
-    glLoadIdentity ();
-    glTranslatef (0.0, 0.0, -DEFAULT_Z_CAMERA);
+    GLMatrix transform;
+    transform.translate (0.0f, 0.0f, -DEFAULT_Z_CAMERA);
 
-    i = optionGetIntensity () * 2;
+    GLfloat ground1Vertices[] =
+    {
+	-0.5, -0.5, 0.0,
+	0.5, -0.5, 0.0,
+	-0.5, 0.0, 0.0,
+	0.5, 0.0, 0.0
+    };
 
-    glBegin (GL_QUADS);
-    glColor4f (0.0, 0.0, 0.0, MAX (0.0, 1.0 - i) );
-    glVertex2f (0.5, 0.0);
-    glVertex2f (-0.5, 0.0);
-    glColor4f (0.0, 0.0, 0.0, MIN (1.0, 1.0 - (i - 1.0) ) );
-    glVertex2f (-0.5, -0.5);
-    glVertex2f (0.5, -0.5);
-    glEnd ();
+    unsigned short maxG1Color = MAX (0.0f, 1.0f - i) * 65535;
+    unsigned short minG1Color = MIN (1.0, 1.0 - (i - 1.0)) * 65535;
+
+    GLushort ground1Colors[] =
+    {
+	0, 0, 0, maxG1Color,
+	0, 0, 0, maxG1Color,
+	0, 0, 0, minG1Color,
+	0, 0, 0, minG1Color
+    };
+
+    GLVertexBuffer *streamingBuffer = GLVertexBuffer::streamingBuffer ();
+    streamingBuffer->begin (GL_TRIANGLE_STRIP);
+
+    streamingBuffer->addVertices (4, ground1Vertices);
+    streamingBuffer->addColors (4, ground1Colors);
+    if (streamingBuffer->end ())
+	streamingBuffer->render (transform);
 
     if (optionGetGroundSize () > 0.0)
     {
-	glBegin (GL_QUADS);
-	glColor4usv (optionGetGroundColor1 () );
-	glVertex2f (-0.5, -0.5);
-	glVertex2f (0.5, -0.5);
-	glColor4usv (optionGetGroundColor2 () );
-	glVertex2f (0.5, -0.5 + optionGetGroundSize () );
-	glVertex2f (-0.5, -0.5 + optionGetGroundSize () );
-	glEnd ();
-    }
+	GLfloat ground2Vertices[] =
+	{
+	    -0.5, -0.5, 0.0,
+	    0.5, -0.5, 0.0,
+	    -0.5, static_cast <GLfloat> (-0.5 + optionGetGroundSize ()), 0.0,
+	    0.5, static_cast <GLfloat> (-0.5 + optionGetGroundSize ()), 0.0
+	};
 
-    glColor4usv (defaultColor);
+	streamingBuffer->begin (GL_TRIANGLE_STRIP);
+	streamingBuffer->addColors (1, optionGetGroundColor1 ());
+	streamingBuffer->addColors (1, optionGetGroundColor1 ());
+	streamingBuffer->addColors (1, optionGetGroundColor2 ());
+	streamingBuffer->addColors (1, optionGetGroundColor2 ());
+	streamingBuffer->addVertices (4, ground2Vertices);
+	if (streamingBuffer->end ())
+	    streamingBuffer->render (transform);
+    }
 
     glBlendFunc (GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
     glDisable (GL_BLEND);
-    glPopMatrix ();
 }
 
 bool 
@@ -257,10 +275,8 @@ CubeaddonScreen::cubeCheckOrientation (const GLScreenPaintAttrib &sAttrib,
 				       CompOutput                *output,
 				       std::vector<GLVector>     &points)
 {
-    bool status;
-
-    status = cubeScreen->cubeCheckOrientation (sAttrib, transform,
-					       output, points);
+    bool status = cubeScreen->cubeCheckOrientation (sAttrib, transform,
+						    output, points);
 
     if (mReflection)
 	return !status;
@@ -296,16 +312,14 @@ CubeaddonScreen::cubeClearTargetOutput (float xRotate, float vRotate)
 	glCullFace (GL_FRONT);
 }
 
-bool 
+bool
 CubeaddonScreen::cubeShouldPaintViewport (const GLScreenPaintAttrib &sAttrib,
 					  const GLMatrix            &transform,
 					  CompOutput                *output,
 					  PaintOrder                order)
 {
-    bool rv = false;
-
-    rv = cubeScreen->cubeShouldPaintViewport (sAttrib, transform,
-					      output, order);
+    bool rv = cubeScreen->cubeShouldPaintViewport (sAttrib, transform,
+						   output, order);
 
     if (rv || cubeScreen->unfolded ())
 	return rv;
@@ -313,15 +327,14 @@ CubeaddonScreen::cubeShouldPaintViewport (const GLScreenPaintAttrib &sAttrib,
     if (mDeform > 0.0 && optionGetDeformation () == DeformationCylinder)
     {
 	float z[3];
-	bool  ftb1, ftb2, ftb3;
 
 	z[0] = cubeScreen->invert () * cubeScreen->distance ();
 	z[1] = z[0] + (0.25 / cubeScreen->distance ());
-	z[2] = cubeScreen->invert () * 
+	z[2] = cubeScreen->invert () *
 	       sqrtf (0.25 + (cubeScreen->distance () * cubeScreen->distance ()));
 
 	std::vector<GLVector> vPoints[3];
-	
+
 	vPoints[0].push_back (GLVector (-0.5,  0.0, z[0], 1.0));
 	vPoints[0].push_back (GLVector ( 0.0,  0.5, z[1], 1.0));
 	vPoints[0].push_back (GLVector ( 0.0,  0.0, z[1], 1.0));
@@ -332,12 +345,12 @@ CubeaddonScreen::cubeShouldPaintViewport (const GLScreenPaintAttrib &sAttrib,
 	vPoints[2].push_back (GLVector ( 0.0,  0.5, z[2], 1.0));
 	vPoints[2].push_back (GLVector ( 0.0,  0.0, z[2], 1.0));
 
-	ftb1 = cubeScreen->cubeCheckOrientation (sAttrib, transform,
-					         output, vPoints[0]);
-	ftb2 = cubeScreen->cubeCheckOrientation (sAttrib, transform,
-					         output, vPoints[1]);
-	ftb3 = cubeScreen->cubeCheckOrientation (sAttrib, transform,
-					         output, vPoints[2]);
+	bool ftb1 = cubeScreen->cubeCheckOrientation (sAttrib, transform,
+						      output, vPoints[0]);
+	bool ftb2 = cubeScreen->cubeCheckOrientation (sAttrib, transform,
+						      output, vPoints[1]);
+	bool ftb3 = cubeScreen->cubeCheckOrientation (sAttrib, transform,
+						      output, vPoints[2]);
 
 	rv = (order == FTB && (ftb1 || ftb2 || ftb3)) ||
 	     (order == BTF && (!ftb1 || !ftb2 || !ftb3));
@@ -345,15 +358,14 @@ CubeaddonScreen::cubeShouldPaintViewport (const GLScreenPaintAttrib &sAttrib,
     else if (mDeform > 0.0 && optionGetDeformation () == DeformationSphere)
     {
 	float z[4];
-	bool  ftb1, ftb2, ftb3, ftb4;
 
 	z[0] = sqrtf (0.5 + (cubeScreen->distance () * cubeScreen->distance ()));
 	z[1] = z[0] + (0.25 / cubeScreen->distance ());
 	z[2] = sqrtf (0.25 + (cubeScreen->distance () * cubeScreen->distance ()));
 	z[3] = z[2] + 0.5;
-	
+
 	std::vector<GLVector> vPoints[4];
-	
+
 	vPoints[0].push_back (GLVector ( 0.0,  0.0, z[3], 1.0));
 	vPoints[0].push_back (GLVector (-0.5,  0.5, z[2], 1.0));
 	vPoints[0].push_back (GLVector ( 0.0,  0.5, z[2], 1.0));
@@ -367,14 +379,14 @@ CubeaddonScreen::cubeShouldPaintViewport (const GLScreenPaintAttrib &sAttrib,
 	vPoints[3].push_back (GLVector ( 0.5,  0.5, z[0], 1.0));
 	vPoints[3].push_back (GLVector ( 0.5,  0.0, z[0], 1.0));
 
-	ftb1 = cubeScreen->cubeCheckOrientation (sAttrib, transform,
-						 output, vPoints[0]);
-	ftb2 = cubeScreen->cubeCheckOrientation (sAttrib, transform,
-						 output, vPoints[1]);
-	ftb3 = cubeScreen->cubeCheckOrientation (sAttrib, transform,
-						 output, vPoints[2]);
-	ftb4 = cubeScreen->cubeCheckOrientation (sAttrib, transform,
-						 output, vPoints[3]);
+	bool ftb1 = cubeScreen->cubeCheckOrientation (sAttrib, transform,
+						      output, vPoints[0]);
+	bool ftb2 = cubeScreen->cubeCheckOrientation (sAttrib, transform,
+						      output, vPoints[1]);
+	bool ftb3 = cubeScreen->cubeCheckOrientation (sAttrib, transform,
+						      output, vPoints[2]);
+	bool ftb4 = cubeScreen->cubeCheckOrientation (sAttrib, transform,
+						      output, vPoints[3]);
 
 	rv = (order == FTB && (ftb1 || ftb2 || ftb3 || ftb4)) ||
 	     (order == BTF && (!ftb1 || !ftb2 || !ftb3 || !ftb4));
@@ -393,7 +405,6 @@ CubeaddonScreen::paintCap (const GLScreenPaintAttrib &sAttrib,
 {
     GLScreenPaintAttrib sa;
     GLMatrix            sTransform;
-    int                 i, l, opacity;
     int                 cullNorm, cullInv;
     bool                wasCulled = glIsEnabled (GL_CULL_FACE);
     float               cInv = (top) ? 1.0: -1.0;
@@ -403,14 +414,14 @@ CubeaddonScreen::paintCap (const GLScreenPaintAttrib &sAttrib,
 
     glGetIntegerv (GL_CULL_FACE_MODE, &cullNorm);
     cullInv   = (cullNorm == GL_BACK)? GL_FRONT : GL_BACK;
+
     if (top)
 	color = cubeScreen->topColor ();
     else
 	color = cubeScreen->bottomColor ();
 
-    opacity = cubeScreen->desktopOpacity () * color[3] / 0xffff;
+    int opacity = cubeScreen->desktopOpacity () * color[3] / 0xffff;
 
-    glPushMatrix ();
     glEnable (GL_BLEND);
 
     if (top)
@@ -424,31 +435,29 @@ CubeaddonScreen::paintCap (const GLScreenPaintAttrib &sAttrib,
 	cAspect = optionGetBottomAspect ();
     }
 
-
-    glDisableClientState (GL_TEXTURE_COORD_ARRAY);
-
-    if (optionGetDeformation () == DeformationSphere &&
-        optionGetDeformCaps ())
-	glEnableClientState (GL_NORMAL_ARRAY);
-
-    glVertexPointer (3, GL_FLOAT, 0, mCapFill);
-
     glEnable(GL_CULL_FACE);
 
-    for (l = 0; l < ((cubeScreen->invert () == 1) ? 2 : 1); l++)
+    for (int l = 0; l < ((cubeScreen->invert () == 1) ? 2 : 1); l++)
     {
-	if (optionGetDeformation () == DeformationSphere &&
-	    optionGetDeformCaps ())
-	{
-	    glNormalPointer (GL_FLOAT, 0, (l == 0) ? mCapFill : mCapFillNorm);
-	}
-	else
-	    glNormal3f (0.0, (l == 0) ? 1.0 : -1.0, 0.0);
-
 	glCullFace(((l == 1) ^ top) ? cullInv : cullNorm);
 
-	for (i = 0; i < size; i++)
+	for (int i = 0; i < size; i++)
 	{
+	    GLVertexBuffer *streamingBuffer = GLVertexBuffer::streamingBuffer ();
+
+	    streamingBuffer->begin (GL_TRIANGLE_FAN);
+
+	    if (optionGetDeformation () == DeformationSphere &&
+		optionGetDeformCaps ())
+	    {
+		streamingBuffer->addNormals (CAP_NVERTEX / 3, (l == 0) ? mCapFill : mCapFillNorm);
+	    }
+	    else
+	    {
+		GLfloat nonDeformNormals[] = { 0.0f, (l == 0) ? 1.0f : -1.0f, 0.0f };
+		streamingBuffer->addNormals (1, nonDeformNormals);
+	    }
+
 	    sa = sAttrib;
 	    sTransform = transform;
 	    if (cubeScreen->invert () == 1)
@@ -468,33 +477,76 @@ CubeaddonScreen::paintCap (const GLScreenPaintAttrib &sAttrib,
 
 	    gScreen->glApplyTransform (sa, output, &sTransform);
 
-	    glLoadMatrixf (sTransform.getMatrix ());
-	    glTranslatef (cubeScreen->outputXOffset (), -cubeScreen->outputYOffset (), 0.0f);
-	    glScalef (cubeScreen->outputXScale (), cubeScreen->outputYScale (), 1.0f);
+	    GLMatrix cTransform (sTransform);
+	    cTransform.translate (cubeScreen->outputXOffset (), -cubeScreen->outputYOffset (), 0.0f);
+	    cTransform.scale (cubeScreen->outputXScale (), cubeScreen->outputYScale (), 1.0f);
+	    cTransform.scale (1.0, cInv, 1.0);
 
-	    glScalef (1.0, cInv, 1.0);
+	    float normalizedOpacity = opacity / static_cast <float> (OPAQUE);
+	    float premultColors[] =
+	    {
+		(color[0] / OPAQUE) * normalizedOpacity,
+		(color[1] / OPAQUE) * normalizedOpacity,
+		(color[2] / OPAQUE) * normalizedOpacity
+	    };
 
-	    glColor4us (color[0] * opacity / 0xffff,
-		color[1] * opacity / 0xffff,
-		color[2] * opacity / 0xffff,
-		opacity);
+	    streamingBuffer->color4f (premultColors[0],
+				      premultColors[1],
+				      premultColors[2],
+				      normalizedOpacity);
+
+	    streamingBuffer->addVertices (CAP_ELEMENTS + 2, mCapFill);
+	    if (streamingBuffer->end ())
+		streamingBuffer->render (cTransform);
 
-	    glDrawArrays (GL_TRIANGLE_FAN, 0, CAP_ELEMENTS + 2);
 	    if (optionGetDeformation () == DeformationSphere &&
 	        optionGetDeformCaps ())
-		glDrawElements (GL_QUADS, CAP_NIDX, GL_UNSIGNED_SHORT,
-				mCapFillIdx);
+	    {
+#ifndef USE_GLES
+		streamingBuffer->begin (GL_QUADS);
+		streamingBuffer->color4f (premultColors[0],
+					  premultColors[1],
+					  premultColors[2],
+					  normalizedOpacity);
+		streamingBuffer->addNormals (CAP_NVERTEX / 3, (l == 0) ? mCapFill : mCapFillNorm);
+
+		GLushort *idx = mCapFillIdx;
+		GLfloat  capVertices[CAP_NIDX * 3];
+		GLfloat  *capVerticesPtr = capVertices;
+
+		for (unsigned int i = 0; i < CAP_NIDX; ++i)
+		{
+		    unsigned int   vertexIndex = idx[i] * 3;
+
+		    *(capVerticesPtr++) = (mCapFill[vertexIndex]);
+		    *(capVerticesPtr++) = (mCapFill[vertexIndex + 1]);
+		    *(capVerticesPtr++) = (mCapFill[vertexIndex + 2]);
+		}
+
+		streamingBuffer->addVertices (CAP_NIDX, &capVertices[0]);
+
+		if (streamingBuffer->end ())
+		    streamingBuffer->render (cTransform);
+#endif
+	    }
 
 	    if (cap->mLoaded)
 	    {
-		float    s_gen[4], t_gen[4];
+		GLVertexBuffer *streamingBuffer = GLVertexBuffer::streamingBuffer ();
 		GLMatrix texMat = cap->mTexMat;
 
 		if (cubeScreen->invert () != 1)
 		    texMat.scale (-1.0, 1.0, 1.0);
 
-		glColor4us (cubeScreen->desktopOpacity (), cubeScreen->desktopOpacity (),
-			    cubeScreen->desktopOpacity (), cubeScreen->desktopOpacity ());
+		streamingBuffer->begin (GL_TRIANGLE_FAN);
+
+		float normalizedOpacity = cubeScreen->desktopOpacity () / OPAQUE;
+
+		streamingBuffer->color4f (normalizedOpacity,
+					  normalizedOpacity,
+					  normalizedOpacity,
+					  normalizedOpacity);
+
 	        cap->mTexture[0]->enable (GLTexture::Good);
 
 		if (cAspect)
@@ -525,86 +577,148 @@ CubeaddonScreen::paintCap (const GLScreenPaintAttrib &sAttrib,
 		
 		texMat.rotate (-(360.0f / size) * i, 0.0, 0.0, 1.0);
 
-		s_gen[0] = texMat[0];
-		s_gen[1] = texMat[8];
-		s_gen[2] = texMat[4];
-		s_gen[3] = texMat[12];
-		t_gen[0] = texMat[1];
-		t_gen[1] = texMat[9];
-		t_gen[2] = texMat[5];
-		t_gen[3] = texMat[13];
+		GLVector sGen (texMat[0], texMat[8], texMat[4], texMat[12]);
+		GLVector tGen (texMat[1], texMat[9], texMat[5], texMat[13]);
 
-		glTexGenfv(GL_T, GL_OBJECT_PLANE, t_gen);
-		glTexGenfv(GL_S, GL_OBJECT_PLANE, s_gen);
+		/* Generate texCoords for the top section of the
+		 * cap */
+		GLfloat texCoords[(CAP_ELEMENTS + 2) * 2];
+		GLfloat *texCoordsPtr = texCoords;
 
-		glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
-		glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
+		for (unsigned int i = 0; i < CAP_ELEMENTS + 2; i++)
+		{
+		    GLVector v (mCapFill[i * 3],
+				mCapFill[i * 3 + 1],
+				mCapFill[i * 3 + 2],
+				1.0f);
+		    float s = v * sGen;
+		    float t = v * tGen;
 
-		glEnable(GL_TEXTURE_GEN_S);
-		glEnable(GL_TEXTURE_GEN_T);
+		    *(texCoordsPtr++) = s;
+		    *(texCoordsPtr++) = t;
+		}
+
+		streamingBuffer->addTexCoords (0, CAP_NVERTEX / 3, &texCoords[0]);
+		streamingBuffer->addVertices (CAP_NVERTEX / 3, mCapFill);
+		streamingBuffer->setMaxVertices (CAP_ELEMENTS + 2);
+		if (streamingBuffer->end ())
+		    streamingBuffer->render (cTransform);
 
-		glDrawArrays (GL_TRIANGLE_FAN, 0, CAP_ELEMENTS + 2);
 		if (optionGetDeformation () == DeformationSphere &&
 	            optionGetDeformCaps ())
-		    glDrawElements (GL_QUADS, CAP_NIDX, GL_UNSIGNED_SHORT,
-				    mCapFillIdx);
+		{
+#ifndef USE_GLES
+		    streamingBuffer->begin (GL_QUADS);
+		    streamingBuffer->color4f (normalizedOpacity,
+					      normalizedOpacity,
+					      normalizedOpacity,
+					      normalizedOpacity);
+		    streamingBuffer->addNormals (CAP_NVERTEX / 3, (l == 0) ? mCapFill : mCapFillNorm);
+
+		    /* Generate texCoords and vertices for the
+		     * curvature around the top section of the cap
+		     *
+		     * This is a little more inefficient than it should be.
+		     *
+		     * The previous implementation used glDrawElements with
+		     * and IBO to ensure that we didn't send lots of redundant
+		     * geometry to the GPU, however GLVertexBuffer doesn't have
+		     * any concept of indexed rendering, and instead takes a vertex
+		     * buffer with all the geometry.
+		     *
+		     * FIXME: This code uses GL_QUADS, so its not compatible with
+		     * OpenGL|ES at the moment
+		     */
+		    GLushort *idx = mCapFillIdx;
+		    GLfloat  capVertices[CAP_NIDX * 3];
+		    GLfloat  texCoords[CAP_NIDX * 2];
+
+		    GLfloat *capVerticesPtr = capVertices;
+		    GLfloat *texCoordsPtr = texCoords;
+
+		    for (unsigned int i = 0; i < CAP_NIDX; ++i)
+		    {
+			unsigned int   vertexIndex = idx[i] * 3;
+
+			GLVector v (mCapFill[vertexIndex],
+				    mCapFill[vertexIndex + 1],
+				    mCapFill[vertexIndex + 2],
+				    1.0f);
+
+			*(capVerticesPtr++) = v[GLVector::x];
+			*(capVerticesPtr++) = v[GLVector::y];
+			*(capVerticesPtr++) = v[GLVector::z];
+
+			/* GL_OBJECT_LINEAR is simulated by doing:
+			 * texCoord.s = dot (vec4 (obj, 1.0), sGenPlane)
+			 * texCoord.t = dot (vec4 (obj, 1.0), tGenPlane)
+			 */
+			float s = v * sGen;
+			float t = v * tGen;
+
+			*(texCoordsPtr++) = s;
+			*(texCoordsPtr++) = t;
+		    }
+
+		    streamingBuffer->addVertices (CAP_NIDX, &capVertices[0]);
+		    streamingBuffer->addTexCoords (0, CAP_NIDX, &texCoords[0]);
+
+		    if (streamingBuffer->end ())
+			streamingBuffer->render (cTransform);
+#endif
+		}
 
-		glDisable(GL_TEXTURE_GEN_S);
-		glDisable(GL_TEXTURE_GEN_T);
 		cap->mTexture[0]->disable ();
 	    }
 	}
     }
 
-    glEnableClientState (GL_TEXTURE_COORD_ARRAY);
-    glDisableClientState (GL_NORMAL_ARRAY);
     glDisable (GL_BLEND);
-    glNormal3f (0.0, -1.0, 0.0);
 
     glCullFace (cullNorm);
     if (!wasCulled)
 	glDisable (GL_CULL_FACE);
-
-    glPopMatrix ();
-
-    glColor4usv (defaultColor);
 }
 
 void 
 CubeaddonScreen::cubePaintTop (const GLScreenPaintAttrib &sAttrib,
 			       const GLMatrix            &transform,
 			       CompOutput                *output,
-			       int                       size)
+			       int                       size,
+			       const GLVector            &normal)
 {
-    if ((!optionGetDrawBottom () && cubeScreen->invert () == -1) ||
-        (!optionGetDrawTop () && cubeScreen->invert () == 1))
-    {
-	cubeScreen->cubePaintTop (sAttrib, transform, output, size);
-    }
-
+    /* we do not want to draw anything if this option is disabled */
     if (!optionGetDrawTop ())
-        return;
+	return;
 
-    paintCap (sAttrib, transform, output, size,
-	      true, optionGetAdjustTop ());
+    if (((!optionGetDrawBottom () && cubeScreen->invert () == -1) ||
+	 (!optionGetDrawTop () && cubeScreen->invert () == 1)) &&
+	/* the original top cap will work for the non-deformed cube only */
+	(optionGetDeformation () == DeformationNone))
+	cubeScreen->cubePaintTop (sAttrib, transform, output, size, normal);
+    else
+	paintCap (sAttrib, transform, output, size,
+		  true, optionGetAdjustTop ());
 }
 
 void 
 CubeaddonScreen::cubePaintBottom (const GLScreenPaintAttrib &sAttrib,
 				  const GLMatrix            &transform,
 				  CompOutput                *output,
-				  int                       size)
+				  int                       size,
+				  const GLVector            &normal)
 {
-    if ((!optionGetDrawBottom () && cubeScreen->invert () == 1) ||
-        (!optionGetDrawTop () && cubeScreen->invert () == -1))
-    {
-	cubeScreen->cubePaintBottom (sAttrib, transform, output, size);
-    }
-
+    /* we do not want to draw anything if this option is disabled */
     if (!optionGetDrawBottom ())
-        return;
+	return;
 
-    paintCap (sAttrib, transform, output, size,
+    if (((!optionGetDrawBottom () && cubeScreen->invert () == 1) ||
+	 (!optionGetDrawTop () && cubeScreen->invert () == -1)) &&
+	/* the original bottom cap will work for the non-deformed cube only */
+	(optionGetDeformation () == DeformationNone))
+	cubeScreen->cubePaintBottom (sAttrib, transform, output, size, normal);
+    else
+	paintCap (sAttrib, transform, output, size,
 	      false, optionGetAdjustBottom ());
 }
 
@@ -617,8 +731,8 @@ CubeaddonWindow::glAddGeometry (const GLTexture::MatrixList &matrix,
 {
     if (caScreen->mDeform > 0.0)
     {
-	GLWindow::Geometry &geometry = gWindow->geometry ();
-	int                i, oldVCount = geometry.vCount;
+	GLVertexBuffer     *vb = gWindow->vertexBuffer ();
+	int                i, oldVCount = vb->countVertices ();
 	GLfloat            *v;
 	int                offX = 0, offY = 0;
 	int                sx1, sx2, sw, sy1, sy2, sh;
@@ -645,10 +759,12 @@ CubeaddonWindow::glAddGeometry (const GLTexture::MatrixList &matrix,
 	gWindow->glAddGeometry (matrix, region, clip, 
 				MIN (CUBEADDON_GRID_SIZE, maxGridWidth),
 				maxGridHeight);
+
+	vb = gWindow->vertexBuffer ();
 	
-	v  = geometry.vertices;
-	v += geometry.vertexStride - 3;
-	v += geometry.vertexStride * oldVCount;
+	v  = vb->getVertices ();
+	v += vb->getVertexStride () - 3;
+	v += vb->getVertexStride () * oldVCount;
 
 	if (!window->onAllViewports ())
 	{
@@ -707,7 +823,7 @@ CubeaddonWindow::glAddGeometry (const GLTexture::MatrixList &matrix,
 	{
 	    float lastX = std::numeric_limits <float>::min (), lastZ = 0.0;
 	
-	    for (i = oldVCount; i < geometry.vCount; i++)
+	    for (i = oldVCount; i < vb->countVertices (); i++)
 	    {
 		if (v[0] == lastX)
 		{
@@ -728,7 +844,7 @@ CubeaddonWindow::glAddGeometry (const GLTexture::MatrixList &matrix,
 		lastX = v[0];
 		lastZ = v[2];
 
-		v += geometry.vertexStride;
+		v += vb->getVertexStride ();
 	    }
 	}
 	else
@@ -738,20 +854,20 @@ CubeaddonWindow::glAddGeometry (const GLTexture::MatrixList &matrix,
 	    last[1][0] = -1000000000.0;
 
 	    int cLast = 0;
-	    for (i = oldVCount; i < geometry.vCount; i++)
+	    for (i = oldVCount; i < vb->countVertices (); i++)
 	    {
 		if (last[0][0] == v[0] && last[0][1] == v[1])
 		{
 		    v[0] = last[0][2];
 		    v[2] = last[0][3];
-		    v += geometry.vertexStride;
+		    v += vb->getVertexStride ();
 		    continue;
 		}
 		else if (last[1][0] == v[0] && last[1][1] == v[1])
 		{
 		    v[0] = last[1][2];
 		    v[2] = last[1][3];
-		    v += geometry.vertexStride;
+		    v += vb->getVertexStride ();
 		    continue;
 		}
 		
@@ -777,7 +893,7 @@ CubeaddonWindow::glAddGeometry (const GLTexture::MatrixList &matrix,
 		    last[cLast][3] = v[2];
 		    cLast = (cLast + 1) & 1;
 		}
-		v += geometry.vertexStride;
+		v += vb->getVertexStride ();
 	    }
 	}
     }
@@ -788,15 +904,14 @@ CubeaddonWindow::glAddGeometry (const GLTexture::MatrixList &matrix,
 }
 
 bool 
-CubeaddonWindow::glDraw (const GLMatrix     &transform,
-			 GLFragment::Attrib &attrib,
-			 const CompRegion   &region,
-			 unsigned int       mask)
+CubeaddonWindow::glDraw (const GLMatrix            &transform,
+			 const GLWindowPaintAttrib &attrib,
+			 const CompRegion          &region,
+			 unsigned int              mask)
 {
     if (!(mask & PAINT_WINDOW_TRANSFORMED_MASK) && caScreen->mDeform)
     {
 	CompPoint offset;
-	int x1, x2;
 
 	if (!window->onAllViewports ())
 	{
@@ -804,8 +919,8 @@ CubeaddonWindow::glDraw (const GLMatrix     &transform,
 	    offset = window->getMovementForOffset (offset);
 	}
 	
-	x1 = window->x () - window->output ().left + offset.x ();
-	x2 = window->x () + window->width () + window->output ().right + offset.x ();
+	int x1 = window->x () - window->output ().left + offset.x ();
+	int x2 = window->x () + window->width () + window->output ().right + offset.x ();
 	if (x1 < 0 && x2 < 0)
 	    return false;
 	if (x1 > screen->width () && x2 > screen->width ())
@@ -816,31 +931,33 @@ CubeaddonWindow::glDraw (const GLMatrix     &transform,
 }
 
 void 
-CubeaddonWindow::glDrawTexture (GLTexture           *texture,
-				GLFragment::Attrib& attrib,
-				unsigned int        mask)
+CubeaddonWindow::glDrawTexture (GLTexture                 *texture,
+				const GLMatrix            &matrix,
+				const GLWindowPaintAttrib &attrib,
+				unsigned int              mask)
 {
     if (caScreen->mDeform > 0.0 && caScreen->gScreen->lighting ())
     {
 	int       i;
 	int       sx1, sx2, sw, sy1, sy2, sh;
 	int       offX = 0, offY = 0;
-	float     x, y, ym;
+	float     x, y;
 	GLfloat   *v, *n;
-	float     inv;
 	
-	GLWindow::Geometry           &geometry = gWindow->geometry ();
+	GLVertexBuffer               *vb = gWindow->vertexBuffer ();
 	CubeScreen::MultioutputMode  cMOM = cubeScreen->multioutputMode ();
 	float                        cDist = cubeScreen->distance ();
 
-	inv = (cubeScreen->invert () == 1) ? 1.0: -1.0;
-	ym  = (caScreen->optionGetDeformation () == CubeaddonScreen::DeformationCylinder) ? 0.0 : 1.0;
+	float inv = (cubeScreen->invert () == 1) ? 1.0: -1.0;
+	float ym  = (caScreen->optionGetDeformation () == CubeaddonScreen::DeformationCylinder) ? 0.0 : 1.0;
 	
-	if ((int) caScreen->mWinNormSize < geometry.vCount * 3)
+	int vertexCount = vb->countVertices ();
+
+	if ((int) caScreen->mWinNormSize < vertexCount * 3)
 	{
 	    delete [] caScreen->mWinNormals;
-	    caScreen->mWinNormals = new GLfloat[geometry.vCount * 3];
-	    caScreen->mWinNormSize = geometry.vCount * 3;
+	    caScreen->mWinNormals = new GLfloat[vertexCount * 3];
+	    caScreen->mWinNormSize = vertexCount * 3;
 	}
 	
 	if (!window->onAllViewports ())
@@ -891,12 +1008,12 @@ CubeaddonWindow::glDrawTexture (GLTexture           *texture,
 	    }
 	}
 	
-	v = geometry.vertices + (geometry.vertexStride - 3);
+	v = vb->getVertices () + (vb->getVertexStride () - 3);
 	n = caScreen->mWinNormals;
 
 	if (cubeScreen->paintOrder () == FTB)
 	{
-	    for (i = 0; i < geometry.vCount; i++)
+	    for (i = 0; i < vertexCount; i++)
 	    {
 		x = (((v[0] + offX - sx1) / (float)sw) - 0.5);
 		y = (((v[1] + offY - sy1) / (float)sh) - 0.5);
@@ -905,12 +1022,12 @@ CubeaddonWindow::glDrawTexture (GLTexture           *texture,
 		*(n)++ = y / sh * caScreen->mDeform * ym;
 		*(n)++ = v[2] + cDist;
 
-		v += geometry.vertexStride;
+		v += vb->getVertexStride ();
 	    }
 	}
 	else
 	{
-	    for (i = 0; i < geometry.vCount; i++)
+	    for (i = 0; i < vertexCount; i++)
 	    {
 		x = (((v[0] + offX - sx1) / (float)sw) - 0.5);
 		y = (((v[1] + offY - sy1) / (float)sh) - 0.5);
@@ -919,24 +1036,18 @@ CubeaddonWindow::glDrawTexture (GLTexture           *texture,
 		*(n)++ = -y / sh * caScreen->mDeform * ym * inv;
 		*(n)++ = -(v[2] + cDist);
     
-		v += geometry.vertexStride;
+		v += vb->getVertexStride ();
 	    }
 	}
-	
-	glEnable (GL_NORMALIZE);
-	glNormalPointer (GL_FLOAT,0, caScreen->mWinNormals);
-	
-	glEnableClientState (GL_NORMAL_ARRAY);
-	
-	gWindow->glDrawTexture (texture, attrib, mask);
 
-	glDisable (GL_NORMALIZE);
-	glDisableClientState (GL_NORMAL_ARRAY);
-	glNormal3f (0.0, 0.0, -1.0);
+	vb->addNormals (caScreen->mWinNormSize / 3,
+			caScreen->mWinNormals);
+
+	gWindow->glDrawTexture (texture, matrix, attrib, mask);
 	return;
     }
 
-    gWindow->glDrawTexture (texture, attrib, mask);
+    gWindow->glDrawTexture (texture, matrix, attrib, mask);
 }
 
 bool
@@ -956,7 +1067,6 @@ CubeaddonScreen::glPaintTransformedOutput (const GLScreenPaintAttrib &sAttrib,
 					   CompOutput                *output,
 					   unsigned int              mask)
 {
-    static GLfloat light0Position[] = { -0.5f, 0.5f, -9.0f, 1.0f };
     GLMatrix       sTransform = transform;
     float          cDist = cubeScreen->distance ();
     float          cDist2 = cubeScreen->distance () * cubeScreen->distance ();
@@ -1048,7 +1158,6 @@ CubeaddonScreen::glPaintTransformedOutput (const GLScreenPaintAttrib &sAttrib,
 	}
 	else
 	{
-	    int i;
 	    float w;
 	    rS = cDist2 + 0.5;
 
@@ -1059,7 +1168,7 @@ CubeaddonScreen::glPaintTransformedOutput (const GLScreenPaintAttrib &sAttrib,
 	    mCapFillNorm[1] = -1.0;
 	    mCapFillNorm[2] = 0.0;
 
-	    for (i = 0; i < CAP_ELEMENTS; i++)
+	    for (int i = 0; i < CAP_ELEMENTS; i++)
 	    {
 		w = (float)(i + 1) / (float)CAP_ELEMENTS;
 
@@ -1170,20 +1279,18 @@ CubeaddonScreen::glPaintTransformedOutput (const GLScreenPaintAttrib &sAttrib,
 		    mYTrans    = 0.0;
 		    if (optionGetDeformation () == DeformationSphere &&
 			optionGetDeformCaps () && optionGetDrawBottom ())
-		    {
 			rYTrans = sqrt (0.5 + cDist2) * -2.0;
-		    }
 		    else
-		    {
 			rYTrans = point[1] * 2.0;
-		    }
+
 		    break;
+
 		case ModeDistance:
 		    mYTrans = 0.0;
 		    rYTrans = sqrt (0.5 + cDist2) * -2.0;
 		    break;
-		default:
 
+		default:
 		    if (optionGetDeformation () == DeformationSphere &&
 			optionGetDeformCaps () && optionGetDrawBottom ())
 		    {
@@ -1204,13 +1311,10 @@ CubeaddonScreen::glPaintTransformedOutput (const GLScreenPaintAttrib &sAttrib,
 		    break;
 	    }
 
-
 	    if (!optionGetAutoZoom () ||
 		((cubeScreen->rotationState () != CubeScreen::RotationManual) &&
 		 optionGetZoomManualOnly ()))
-	    {
 		mZTrans = 0.0;
-	    }
 	    else
 		mZTrans = -point2[2] + cDist;
 
@@ -1257,82 +1361,95 @@ CubeaddonScreen::glPaintTransformedOutput (const GLScreenPaintAttrib &sAttrib,
 		rTransform.scale (1.0, -1.0, 1.0);
 	    }
 
-	    glPushMatrix ();
-	    glLoadIdentity ();
-	    glScalef (1.0, -1.0, 1.0);
-	    glLightfv (GL_LIGHT0, GL_POSITION, light0Position);
-	    glPopMatrix ();
 	    glCullFace (GL_FRONT);
 
 	    gScreen->glPaintTransformedOutput (sAttrib, rTransform,
 					       region, output, mask);
 
 	    glCullFace (GL_BACK);
-	    glPushMatrix ();
-	    glLoadIdentity ();
-	    glLightfv (GL_LIGHT0, GL_POSITION, light0Position);
-	    glPopMatrix ();
 
 	    if (optionGetMode () == ModeAbove && mVRot > 0.0)
 	    {
-		int   j;
-		float i, c;
 		float v = MIN (1.0, mVRot / 30.0);
-		float col1[4], col2[4];
+		unsigned short col1[4], col2[4];
 
-		glPushMatrix ();
+		GLVertexBuffer *streamingBuffer = GLVertexBuffer::streamingBuffer ();
+
+		GLMatrix gTransform;
 
 		glEnable (GL_BLEND);
 		glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 
-		glLoadIdentity ();
-		glTranslatef (0.0, 0.0, -DEFAULT_Z_CAMERA);
+		gTransform.translate (0, 0, -DEFAULT_Z_CAMERA);
 
-		i = optionGetIntensity () * 2;
-		c = optionGetIntensity ();
+		float i = optionGetIntensity () * 2;
+		float c = optionGetIntensity ();
 
-		glBegin (GL_QUADS);
-		glColor4f (0.0, 0.0, 0.0,
-			   ((1 - v) * MAX (0.0, 1.0 - i)) + (v * c));
-		glVertex2f (0.5, v / 2.0);
-		glVertex2f (-0.5, v / 2.0);
-		glColor4f (0.0, 0.0, 0.0,
-			   ((1 - v) * MIN (1.0, 1.0 - (i - 1.0))) + (v * c));
-		glVertex2f (-0.5, -0.5);
-		glVertex2f (0.5, -0.5);
-		glEnd ();
+		GLfloat vertices[] =
+		{
+		    0.5f, v / 2.0f, 0.0f,
+		    -0.5f, v / 2.0f, 0.0f,
+		    -0.5f, -0.5f, 0.0f,
+		    0.5f, -0.5f, 0.0f
+		};
 
-		for (j = 0; j < 4; j++)
+		unsigned short cMax = MAX (0.0, 1.0 - i) + (v * c);
+		unsigned short cMin = MIN (1.0, 1.0 - (i - 1.0)) + (v * c);
+
+		GLushort colors[] =
+		{
+		    0, 0, 0, cMax,
+		    0, 0, 0, cMax,
+		    0, 0, 0, cMin,
+		    0, 0, 0, cMin
+		};
+
+		streamingBuffer->begin (GL_TRIANGLE_STRIP);
+		streamingBuffer->addColors (4, colors);
+		streamingBuffer->addVertices (4, vertices);
+		if (streamingBuffer->end ())
+		    streamingBuffer->render (gTransform);
+
+		for (int j = 0; j < 4; j++)
 		{
 		    col1[j] = (1.0 - v) * optionGetGroundColor1 () [j] +
 			      (v * (optionGetGroundColor1 () [j] +
 				    optionGetGroundColor2 () [j]) * 0.5);
-		    col1[j] /= 0xffff;
 		    col2[j] = (1.0 - v) * optionGetGroundColor2 () [j] +
 			      (v * (optionGetGroundColor1 () [j] +
 				    optionGetGroundColor2 () [j]) * 0.5);
-		    col2[j] /= 0xffff;
 		}
 
 		if (optionGetGroundSize () > 0.0)
 		{
-		    glBegin (GL_QUADS);
-		    glColor4fv (col1);
-		    glVertex2f (-0.5, -0.5);
-		    glVertex2f (0.5, -0.5);
-		    glColor4fv (col2);
-		    glVertex2f (0.5, -0.5 +
-				((1 - v) * optionGetGroundSize ()) + v);
-		    glVertex2f (-0.5, -0.5 +
-				((1 - v) * optionGetGroundSize ()) + v);
-		    glEnd ();
-		}
+		    GLfloat vertices[] =
+		    {
+			-0.5f, -0.5f, 0.0f,
+			0.5f, -0.5f, 0.0f,
+			0.5f, -0.5f +
+			static_cast <GLfloat> (((1 - v) * optionGetGroundSize ()) + v),
+			-0.5f, -0.5f +
+			static_cast <GLfloat> (((1 - v) * optionGetGroundSize ()) + v)
+		    };
 
-		glColor4usv (defaultColor);
+		    GLushort colors[] =
+		    {
+			col1[0], col1[1], col1[2], col1[3],
+			col1[0], col1[1], col1[2], col1[3],
+			col2[0], col2[1], col2[2], col2[3],
+			col2[0], col2[1], col2[2], col2[3]
+		    };
+
+		    streamingBuffer->begin (GL_TRIANGLE_STRIP);
+		    streamingBuffer->addVertices (4, vertices);
+		    streamingBuffer->addColors (4, colors);
+
+		    if (streamingBuffer->end ())
+			streamingBuffer->render (gTransform);
+		}
 
 		glBlendFunc (GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
 		glDisable (GL_BLEND);
-		glPopMatrix ();
 	    }
 	    else
 		drawBasicGround ();
@@ -1390,7 +1507,6 @@ CubeaddonScreen::donePaint ()
     cScreen->donePaint ();
 }
 
-
 CubeaddonScreen::CubeaddonScreen (CompScreen *s) :
     PluginClassHandler<CubeaddonScreen, CompScreen> (s),
     CubeaddonOptions (),
@@ -1412,7 +1528,6 @@ CubeaddonScreen::CubeaddonScreen (CompScreen *s) :
 
     idx = mCapFillIdx;
     for (int i = 0; i < CAP_ELEMENTS - 1; i++)
-    {
 	for (int j = 0; j < CAP_ELEMENTS; j++)
 	{
 	    idx[0] = 1 + (i * (CAP_ELEMENTS + 1)) + j;
@@ -1421,7 +1536,6 @@ CubeaddonScreen::CubeaddonScreen (CompScreen *s) :
 	    idx[3] = 2 + (i * (CAP_ELEMENTS + 1)) + j;
 	    idx += 4;
 	}
-    }
 
     mTopCap.mFiles = optionGetTopImages ();
     mBottomCap.mFiles = optionGetBottomImages ();
@@ -1474,12 +1588,11 @@ CubeaddonWindow::CubeaddonWindow (CompWindow *w) :
 bool
 CubeaddonPluginVTable::init ()
 {
-    if (!CompPlugin::checkPluginABI ("core", CORE_ABIVERSION) ||
-	!CompPlugin::checkPluginABI ("composite", COMPIZ_COMPOSITE_ABI) ||
-	!CompPlugin::checkPluginABI ("opengl", COMPIZ_OPENGL_ABI) ||
-	!CompPlugin::checkPluginABI ("cube", COMPIZ_CUBE_ABI))
-	return false;
+    if (CompPlugin::checkPluginABI ("core", CORE_ABIVERSION)		&&
+	CompPlugin::checkPluginABI ("composite", COMPIZ_COMPOSITE_ABI)	&&
+	CompPlugin::checkPluginABI ("opengl", COMPIZ_OPENGL_ABI)	&&
+	CompPlugin::checkPluginABI ("cube", COMPIZ_CUBE_ABI))
+	return true;
 
-    return true;
+    return false;
 }
-
