@@ -42,6 +42,36 @@
 
 COMPIZ_PLUGIN_20090315 (decor, DecorPluginVTable)
 
+namespace cwe = compiz::window::extents;
+
+namespace
+{
+void updateRegionWithShapeRectangles (Display    *dpy,
+				      Window     w,
+				      CompRegion &region)
+{
+    int n = 0;
+    int order = 0;
+    XRectangle *shapeRects = NULL;
+
+    shapeRects =
+	XShapeGetRectangles (dpy,
+	    w, ShapeInput,
+	    &n, &order);
+    if (!shapeRects)
+	return;
+
+    for (int i = 0; i < n; i++)
+	region +=
+	    CompRegion (shapeRects[i].x,
+			shapeRects[i].y,
+			shapeRects[i].width,
+			shapeRects[i].height);
+
+    XFree (shapeRects);
+}
+}
+
 MatchedDecorClipGroup::MatchedDecorClipGroup (const CompMatch &match) :
     mMatch (match)
 {
@@ -391,10 +421,9 @@ DecorScreen::getTexture (Pixmap pixmap)
 	    return t;
 	}
 
-    X11PixmapDeletor::Ptr dl = boost::make_shared <X11PixmapDeletor> (screen->dpy ());
-    DecorPixmap::Ptr pm = boost::make_shared <DecorPixmap> (pixmap, dl);
+    DecorPixmap::Ptr pm = boost::make_shared <DecorPixmap> (pixmap, mReleasePool);
 
-    DecorTexture *texture = new DecorTexture (boost::shared_static_cast <DecorPixmapInterface> (pm));
+    DecorTexture *texture = new DecorTexture (pm);
 
     if (!texture->status)
     {
@@ -597,6 +626,7 @@ Decoration::Decoration (int   type,
 			unsigned int nQuad,
 			Window       owner,
 			DecorPixmapRequestorInterface *requestor) :
+    refCount (0),
     texture (DecorScreen::get (screen)->getTexture (pixmap)),
     border (border.left, border.right, border.top, border.bottom),
     input (input.left, input.right, input.top, input.bottom),
@@ -855,7 +885,7 @@ DecorationList::updateDecoration (Window   id,
 	    /* Creating a new decoration failed ... see if we can use
 	     * the old one */
 
-	    unsigned int    frameType, frameState, frameActions;
+	    unsigned int    frameType, frameState, frameActions = 0;
 	    Pixmap	    pixmap = None;
 	    decor_extents_t border;
 	    decor_extents_t input;
@@ -940,6 +970,7 @@ WindowDecoration *
 WindowDecoration::create (const Decoration::Ptr &d)
 {
     WindowDecoration *wd;
+    unsigned int     nQuad = d->nQuad;
 
     wd = new WindowDecoration ();
     if (!wd)
@@ -947,7 +978,7 @@ WindowDecoration::create (const Decoration::Ptr &d)
 
     if (d->type == WINDOW_DECORATION_TYPE_PIXMAP)
     {
-	wd->quad = new ScaledQuad[d->nQuad];
+	wd->quad = new ScaledQuad[nQuad];
 
 	if (!wd->quad)
 	{
@@ -956,12 +987,15 @@ WindowDecoration::create (const Decoration::Ptr &d)
 	}
     }
     else
+    {
+	nQuad = 0;
 	wd->quad = NULL;
+    }
 
     d->refCount++;
 
     wd->decor = d;
-    wd->nQuad = d->nQuad;
+    wd->nQuad = nQuad;
 
     return wd;
 }
@@ -997,7 +1031,6 @@ WindowDecoration::destroy (WindowDecoration *wd)
 void
 DecorWindow::setDecorationMatrices ()
 {
-    int		      i;
     float	      x0, y0;
     decor_matrix_t    a;
     GLTexture::Matrix b;
@@ -1005,7 +1038,7 @@ DecorWindow::setDecorationMatrices ()
     if (!wd)
 	return;
 
-    for (i = 0; i < wd->nQuad; i++)
+    for (int i = 0; i < wd->nQuad; i++)
     {
 	/* Set the quad matrix to the texture matrix */
 	wd->quad[i].matrix = wd->decor->texture->textures[0]->matrix ();
@@ -1081,12 +1114,11 @@ DecorWindow::updateDecorationScale ()
 {
     int		     x1, y1, x2, y2;
     float            sx, sy;
-    int		     i;
 
     if (!wd)
 	return;
 
-    for (i = 0; i < wd->nQuad; i++)
+    for (int i = 0; i < wd->nQuad; i++)
     {
 	int x, y;
 	unsigned int width = window->size ().width ();
@@ -1142,31 +1174,6 @@ DecorWindow::checkSize (const Decoration::Ptr &decoration)
 	    decoration->minHeight <= (int) window->size ().height ());
 }
 
-/*
- * decorOffsetMove
- *
- * Function called on a timer (to avoid calling configureXWindow from
- * within a ::moveNotify) which actually moves the window by the offset
- * specified in the xwc. Also sends a notification that the window
- * was decorated
- *
- */
-static bool
-decorOffsetMove (CompWindow *w, XWindowChanges xwc, unsigned int mask)
-{
-    CompOption::Vector o (1);
-
-    o.at (0).setName ("window", CompOption::TypeInt);
-    o.at (0).value ().set ((int) w->id ());
-
-    xwc.x += w->serverGeometry ().x ();
-    xwc.y += w->serverGeometry ().y ();
-
-    w->configureXWindow (mask, &xwc);
-    screen->handleCompizEvent ("decor", "window_decorated", o);
-    return false;
-}
-
 /*
  * DecorWindow::matchType
  *
@@ -1179,7 +1186,6 @@ DecorWindow::matchType (CompWindow *w,
                         unsigned int decorType)
 {
     const unsigned int nTypeStates = 5;
-    unsigned int i;
     struct typestate {
         unsigned int compFlag;
         unsigned int decorFlag;
@@ -1192,7 +1198,7 @@ DecorWindow::matchType (CompWindow *w,
         { CompWindowTypeUtilMask, DECOR_WINDOW_TYPE_UTILITY}
     };
 
-    for (i = 0; i < nTypeStates; i++)
+    for (unsigned int i = 0; i < nTypeStates; i++)
     {
         if ((decorType & typeStates[i].decorFlag) && (w->type () & typeStates[i].compFlag))
             return true;
@@ -1217,7 +1223,6 @@ DecorWindow::matchState (CompWindow   *w,
                          unsigned int decorState)
 {
     const unsigned int nStateStates = 3;
-    unsigned int i;
     struct statestate {
         unsigned int compFlag;
         unsigned int decorFlag;
@@ -1232,7 +1237,7 @@ DecorWindow::matchState (CompWindow   *w,
     if (screen->activeWindow () == w->id ())
         decorState &= ~(DECOR_WINDOW_STATE_FOCUS);
 
-    for (i = 0; i < nStateStates; i++)
+    for (unsigned int i = 0; i < nStateStates; i++)
     {
         if ((decorState & stateStates[i].decorFlag) && (w->state () & stateStates[i].compFlag))
             decorState &= ~(stateStates[i].decorFlag);
@@ -1253,7 +1258,6 @@ DecorWindow::matchActions (CompWindow   *w,
                            unsigned int decorActions)
 {
     const unsigned int nActionStates =16;
-    unsigned int i;
     struct actionstate {
         unsigned int compFlag;
         unsigned int decorFlag;
@@ -1277,7 +1281,7 @@ DecorWindow::matchActions (CompWindow   *w,
         { DECOR_WINDOW_ACTION_BELOW, CompWindowActionBelowMask },
     };
 
-    for (i = 0; i < nActionStates; i++)
+    for (unsigned int i = 0; i < nActionStates; i++)
     {
         if ((decorActions & actionStates[i].decorFlag) && (w->type () & actionStates[i].compFlag))
             decorActions &= ~(actionStates[i].decorFlag);
@@ -1289,14 +1293,26 @@ DecorWindow::matchActions (CompWindow   *w,
 DecorationInterface::Ptr
 DecorationList::findMatchingDecoration (unsigned int frameType,
 					unsigned int frameState,
-					unsigned int frameActions)
+					unsigned int frameActions) const
 {
     foreach (const Decoration::Ptr &d, mList)
     {
 	if (d->frameType == frameType &&
 	    d->frameState == frameState &&
 	    d->frameActions == frameActions)
-	    return boost::shared_static_cast <DecorationInterface> (d);
+	    return d;
+    }
+
+    return DecorationInterface::Ptr ();
+}
+
+DecorationInterface::Ptr
+DecorationList::findMatchingDecoration (Pixmap p) const
+{
+    foreach (const Decoration::Ptr &d, mList)
+    {
+	if (d->texture->pixmap->getPixmap () == p)
+	    return d;
     }
 
     return DecorationInterface::Ptr ();
@@ -1388,6 +1404,143 @@ DecorationList::findMatchingDecoration (CompWindow *w,
     return *cit;
 }
 
+bool
+DecorWindow::bareDecorationOnly ()
+{
+    bool shadowOnly = true;
+    /* Only want to decorate windows which have a frame or are in the process
+     * of waiting for an animation to be unmapped (in which case we can give
+     * them a new pixmap type frame since we don't actually need an input
+     * window to go along with that
+     *
+     * FIXME: That's not going to play nice with reparented decorations in core
+     * since the window gets reparented right away before plugins are done
+     * with it */
+
+    /* Unconditionally decorate switchers */
+    if (!isSwitcher)
+    {
+        switch (window->type ()) {
+	    case CompWindowTypeDialogMask:
+	    case CompWindowTypeModalDialogMask:
+	    case CompWindowTypeUtilMask:
+	    case CompWindowTypeMenuMask:
+	    case CompWindowTypeNormalMask:
+		if (window->mwmDecor () & (MwmDecorAll | MwmDecorTitle))
+		    shadowOnly = false;
+	    default:
+		break;
+	}
+
+	if (window->overrideRedirect ())
+	    shadowOnly = true;
+
+	if (window->wmType () & (CompWindowTypeDockMask | CompWindowTypeDesktopMask))
+	    shadowOnly = true;
+
+	if (!shadowOnly)
+	{
+	    if (!dScreen->optionGetDecorationMatch ().evaluate (window))
+		shadowOnly = true;
+	}
+
+	/* Never on unmapped windows */
+	if (!window->isViewable ())
+	    shadowOnly = false;
+    }
+    else
+	shadowOnly = false;
+
+    return shadowOnly;
+}
+
+Decoration::Ptr
+DecorWindow::findRealDecoration ()
+{
+    Decoration::Ptr decoration;
+
+    /* Attempt to find a matching decoration */
+    try
+    {
+	decoration = decor.findMatchingDecoration (window, true);
+    }
+    catch (...)
+    {
+	/* Find an appropriate default decoration to use */
+	if (dScreen->dmSupports & WINDOW_DECORATION_TYPE_PIXMAP &&
+	    dScreen->cmActive &&
+	    !(dScreen->dmSupports & WINDOW_DECORATION_TYPE_WINDOW &&
+	    pixmapFailed))
+	{
+	    try
+	    {
+		decoration = dScreen->decor[DECOR_ACTIVE].findMatchingDecoration (window, false);
+	    }
+	    catch (...)
+	    {
+		compLogMessage ("decor", CompLogLevelWarn, "No default decoration found, placement will not be correct");
+		decoration.reset ();
+	    }
+	}
+	else if (dScreen->dmSupports & WINDOW_DECORATION_TYPE_WINDOW)
+	    decoration = dScreen->windowDefault;
+    }
+
+    return decoration;
+}
+
+Decoration::Ptr
+DecorWindow::findBareDecoration ()
+{
+    Decoration::Ptr decoration;
+    /* This window isn't "decorated" but it still gets a shadow as long
+     * as it isn't shaped weirdly, since the shadow is just a quad rect */
+    if (dScreen->optionGetShadowMatch ().evaluate (window))
+    {
+	if (window->region ().numRects () == 1 && !window->alpha () && dScreen->decor[DECOR_BARE].mList.size ())
+	    decoration = dScreen->decor[DECOR_BARE].mList.front ();
+
+	if (decoration)
+	{
+	    if (!checkSize (decoration))
+		decoration.reset ();
+	}
+    }
+
+    return decoration;
+}
+
+namespace
+{
+bool
+shouldDecorateWindow (CompWindow *w,
+                      bool       shadowOnly,
+                      bool       isSwitcher)
+{
+    const bool frameOrUnmapReference = (w->frame () ||
+		          w->hasUnmapReference ());
+    const bool realDecoration = frameOrUnmapReference && !shadowOnly;
+    const bool forceDecoration = isSwitcher;
+
+    return realDecoration || forceDecoration;
+}
+
+/*
+ * notifyDecoration
+ *
+ * Notify other plugins that the window is now fully decorated
+ */
+void notifyDecoration (CompWindow *window)
+{
+    CompOption::Vector o (1);
+
+    o.at (0).setName ("window", CompOption::TypeInt);
+    o.at (0).value ().set ((int) window->id ());
+
+    screen->handleCompizEvent ("decor", "window_decorated", o);
+}
+}
+
 /*
  * DecorWindow::update
  * This is the master function for managing decorations on windows
@@ -1428,108 +1581,28 @@ bool
 DecorWindow::update (bool allowDecoration)
 {
     Decoration::Ptr  old, decoration;
-    bool	     decorate = false;
-    bool	     shadowOnly = true;
-    CompPoint        oldShift, movement;
+    CompPoint        movement;
+    CompSize         sizeDelta;
 
     if (wd)
 	old = wd->decor;
     else
 	old.reset ();
 
-    /* Only want to decorate windows which have a frame or are in the process
-     * of waiting for an animation to be unmapped (in which case we can give
-     * them a new pixmap type frame since we don't actually need an input
-     * window to go along with that
-     *
-     * FIXME: That's not going to play nice with reparented decorations in core
-     * since the window gets reparented right away before plugins are done
-     * with it */
-
-    /* Unconditionally decorate switchers */
-    if (!isSwitcher)
-    {
-        switch (window->type ()) {
-	    case CompWindowTypeDialogMask:
-	    case CompWindowTypeModalDialogMask:
-	    case CompWindowTypeUtilMask:
-	    case CompWindowTypeMenuMask:
-	    case CompWindowTypeNormalMask:
-		if (window->mwmDecor () & (MwmDecorAll | MwmDecorTitle))
-		    shadowOnly = false;
-	    default:
-		break;
-	}
-
-	if (window->overrideRedirect ())
-	    shadowOnly = true;
-
-	if (window->wmType () & (CompWindowTypeDockMask | CompWindowTypeDesktopMask))
-	    shadowOnly = true;
-
-	if (!shadowOnly)
-	{
-	    if (!dScreen->optionGetDecorationMatch ().evaluate (window))
-		shadowOnly = true;
-	}
-    }
-    else
-	shadowOnly = false;
-
-    decorate = ((window->frame () ||
-		 window->hasUnmapReference ()) && !shadowOnly) ||
-		 isSwitcher;
+    bool shadowOnly = bareDecorationOnly ();
+    bool decorate = shouldDecorateWindow (window, shadowOnly, isSwitcher);
+    unsigned int decorMaximizeState = window->state () & MAXIMIZE_STATE;
 
     if (decorate || frameExtentsRequested)
     {
-        /* Attempt to find a matching decoration */
-	try
-	{
-	    decoration = decor.findMatchingDecoration (window, true);
-	}
-	catch (...)
-	{
-	    /* Find an appropriate default decoration to use */
-	    if (dScreen->dmSupports & WINDOW_DECORATION_TYPE_PIXMAP &&
-	        dScreen->cmActive &&
-		!(dScreen->dmSupports & WINDOW_DECORATION_TYPE_WINDOW &&
-		  pixmapFailed))
-	    {
-		try
-		{
-		    decoration = dScreen->decor[DECOR_ACTIVE].findMatchingDecoration (window, false);
-		}
-		catch (...)
-		{
-		    compLogMessage ("decor", CompLogLevelWarn, "No default decoration found, placement will not be correct");
-		    decoration.reset ();
-		}
-	    }
-	    else if (dScreen->dmSupports & WINDOW_DECORATION_TYPE_WINDOW)
-		decoration = dScreen->windowDefault;
-	}
-
+	decoration = findRealDecoration ();
 	/* Do not allow windows which are later undecorated
 	 * to have a set _NET_FRAME_EXTENTS */
 	if (decorate)
 	    frameExtentsRequested = false;
     }
     else
-    {
-	/* This window isn't "decorated" but it still gets a shadow as long
-	 * as it isn't shaped weirdly, since the shadow is just a quad rect */
-	if (dScreen->optionGetShadowMatch ().evaluate (window))
-	{
-	    if (window->region ().numRects () == 1 && !window->alpha () && dScreen->decor[DECOR_BARE].mList.size ())
-		decoration = dScreen->decor[DECOR_BARE].mList.front ();
-
-	    if (decoration)
-	    {
-		if (!checkSize (decoration))
-		    decoration.reset ();
-	    }
-	}
-    }
+    	decoration = findBareDecoration ();
 
     /* Don't allow the windows to be decorated if
      * we're tearing down or if a decorator isn't running
@@ -1541,9 +1614,83 @@ DecorWindow::update (bool allowDecoration)
     /* Don't bother going any further if
      * this window is going to get the same
      * decoration, just use the old one */
-    if (decoration == old)
+    if (decoration == old &&
+        lastMaximizedStateDecorated == decorMaximizeState)
 	return false;
 
+    /* If a decoration was found for this window, create
+     * a new WindowDecoration for it and set the frame
+     * extents accordingly. We should also move the
+     * window by the distance the new decoration provides
+     * in case that actually changed
+     */
+    if (decoration)
+    {
+	/* Set extents based on maximize/unmaximize state
+	 * FIXME: With the new type system, this should be
+	 * removed */
+	if (decorMaximizeState == MAXIMIZE_STATE)
+	    window->setWindowFrameExtents (&decoration->maxBorder,
+					   &decoration->maxInput);
+	else if (!window->hasUnmapReference ())
+	    window->setWindowFrameExtents (&decoration->border,
+					   &decoration->input);
+
+	lastMaximizedStateDecorated = decorMaximizeState;
+
+	/* This window actually needs its decoration contents updated
+	 * as it was actually visible */
+	if (decorate ||
+	    shadowOnly)
+	{
+	    if (wd)
+		WindowDecoration::destroy (wd);
+	    wd = WindowDecoration::create (decoration);
+	    if (!wd)
+	    {
+		/* Error condition, reset frame extents */
+		CompWindowExtents emptyExtents;
+		memset (&emptyExtents, 0, sizeof (CompWindowExtents));
+		window->setWindowFrameExtents (&emptyExtents, &emptyExtents);
+		return false;
+	    }
+
+	    window->updateWindowOutputExtents ();
+
+	    updateReg = true;
+	    updateMatrix = true;
+	    mOutputRegion = CompRegion (window->outputRect ());
+	    if (dScreen->cmActive)
+		cWindow->damageOutputExtents ();
+	    updateDecorationScale ();
+
+	    /* Update the input and output frame */
+	    if (decorate)
+		updateFrame ();
+	}
+    }
+    else
+    {
+	CompWindowExtents emptyExtents;
+
+	if (wd)
+	{
+	    WindowDecoration::destroy (wd);
+	    wd = NULL;
+	}
+
+	/* _NET_FRAME_EXTENTS should be updated before the frame
+	 * atom is */
+	memset (&emptyExtents, 0, sizeof (CompWindowExtents));
+
+	window->setWindowFrameExtents (&emptyExtents, &emptyExtents);
+
+	/* Undecorated windows need to have the
+	 * input and output frame removed and the
+	 * frame window geometry reset */
+	updateFrame ();
+    }
+
     /* We need to damage the current output extents
      * and recompute the shadow region if a compositor
      * is running
@@ -1554,121 +1701,7 @@ DecorWindow::update (bool allowDecoration)
 	updateGroupShadows ();
     }
 
-    /* Determine how much we moved the window for the old
-     * decoration and save that, also destroy the old
-     * WindowDecoration */
-    if (old)
-    {
-	oldShift = compiz::window::extents::shift (window->border (), window->sizeHints ().win_gravity);
-
-	WindowDecoration::destroy (wd);
-
-	wd = NULL;
-    }
-
-    /* If a decoration was found for this window, create
-     * a new WindowDecoration for it and set the frame
-     * extents accordingly. We should also move the
-     * window by the distance the new decoration provides
-     * in case that actually changed
-     */
-    if (decoration)
-    {
-	wd = WindowDecoration::create (decoration);
-	if (!wd)
-	    return false;
-
-	/* Set extents based on maximize/unmaximize state
-	 * FIXME: With the new type system, this should be
-	 * removed */
-	if ((window->state () & MAXIMIZE_STATE))
-	    window->setWindowFrameExtents (&wd->decor->maxBorder,
-					   &wd->decor->maxInput);
-	else if (!window->hasUnmapReference ())
-	    window->setWindowFrameExtents (&wd->decor->border,
-					   &wd->decor->input);
-
-	movement = compiz::window::extents::shift (window->border (), window->sizeHints ().win_gravity);
-	movement -= oldShift;
-
-	/* Update the input and output frame */
-	if (decorate)
-	    updateFrame ();
-	window->updateWindowOutputExtents ();
-
-	updateReg = true;
-	updateMatrix = true;
-	mOutputRegion = CompRegion (window->outputRect ());
-	updateGroupShadows ();
-	if (dScreen->cmActive)
-	    cWindow->damageOutputExtents ();
-	updateDecorationScale ();
-    }
-    else
-    {
-	CompWindowExtents emptyExtents;
-	wd = NULL;
-
-	/* Undecorated windows need to have the
-	 * input and output frame removed and the
-	 * frame window geometry reset */
-	updateFrame ();
-
-	memset (&emptyExtents, 0, sizeof (CompWindowExtents));
-
-	window->setWindowFrameExtents (&emptyExtents, &emptyExtents);
-
-	movement -= oldShift;
-    }
-
-    /* Need to actually move the window */
-    if (window->placed () && !window->overrideRedirect () &&
-	(movement.x () || movement.y ()))
-    {
-	XWindowChanges xwc;
-	unsigned int   mask = CWX | CWY;
-
-	memset (&xwc, 0, sizeof (XWindowChanges));
-
-	/* Grab the geometry last sent to server at configureXWindow
-	 * time and not here since serverGeometry may be updated by
-	 * the time that we do call configureXWindow */
-	xwc.x = movement.x ();
-	xwc.y = movement.y ();
-
-	/* Except if it's fullscreen, maximized or such */
-	if (window->state () & CompWindowStateFullscreenMask)
-	    mask &= ~(CWX | CWY);
-
-	if (window->state () & CompWindowStateMaximizedHorzMask)
-	    mask &= ~CWX;
-
-	if (window->state () & CompWindowStateMaximizedVertMask)
-	    mask &= ~CWY;
-
-	if (window->saveMask () & CWX)
-	    window->saveWc ().x += movement.x ();
-
-	if (window->saveMask () & CWY)
-	    window->saveWc ().y += movement.y ();
-
-	if (mask)
-	{
-	    /* allowDecoration is only false in the case of
-	     * the destructor calling the update function so since it
-	     * is not safe to put the function in a timer (since
-	     * it will get unref'd on the vtable destruction) we
-	     * need to do it immediately
-	     *
-	     * FIXME: CompTimer should really be PIMPL and allow
-	     * refcounting in case we need to keep it alive
-	     */
-	    if (!allowDecoration)
-		decorOffsetMove (window, xwc, mask);
-	    else
-		moveUpdate.start (boost::bind (decorOffsetMove, window, xwc, mask), 0);
-	}
-    }
+    notifyDecoration (window);
 
     return true;
 }
@@ -1774,7 +1807,7 @@ DecorWindow::updateInputFrame ()
 	parent = window->frame ();
 
     /* Determine frame extents */
-    if ((window->state () & MAXIMIZE_STATE))
+    if ((window->state () & MAXIMIZE_STATE) == MAXIMIZE_STATE)
     {
 	border = wd->decor->maxBorder;
 	input = wd->decor->maxInput;
@@ -1898,6 +1931,13 @@ DecorWindow::updateInputFrame ()
 				 ShapeSet, YXBanded);
 
 	frameRegion = CompRegion ();
+
+	/* Immediately query shape rectangles so that we can
+	 * report them if core asks for them */
+	updateRegionWithShapeRectangles (screen->dpy (),
+					 inputFrame,
+					 frameRegion);
+	window->updateFrameRegion ();
     }
 
     XUngrabServer (screen->dpy ());
@@ -1930,7 +1970,7 @@ DecorWindow::updateOutputFrame ()
     CompWindowExtents	 input;
 
     /* Determine frame extents */
-    if ((window->state () & MAXIMIZE_STATE))
+    if ((window->state () & MAXIMIZE_STATE) == MAXIMIZE_STATE)
 	input = wd->decor->maxInput;
     else
 	input = wd->decor->input;
@@ -2040,6 +2080,13 @@ DecorWindow::updateOutputFrame ()
 				 ShapeSet, YXBanded);
 
 	frameRegion = CompRegion ();
+
+	/* Immediately query shape rectangles so that we can
+	 * report them if core asks for them */
+	updateRegionWithShapeRectangles (screen->dpy (),
+					 outputFrame,
+					 frameRegion);
+	window->updateFrameRegion ();
     }
 
     XUngrabServer (screen->dpy ());
@@ -2163,6 +2210,7 @@ void
 DecorWindow::updateFrameRegion (CompRegion &region)
 {
     window->updateFrameRegion (region);
+
     if (wd)
     {
 	if (!frameRegion.isEmpty ())
@@ -2175,10 +2223,6 @@ DecorWindow::updateFrameRegion (CompRegion &region)
 	    region += frameRegion.translated (x - wd->decor->input.left,
 					      y - wd->decor->input.top);
 	}
-	else
-	{
-	    region += infiniteRegion;
-	}
     }
 
     updateReg = true;
@@ -2208,6 +2252,20 @@ DecorWindow::updateWindowRegions ()
     updateReg = false;
 }
 
+/*
+ * DecorWindow::place
+ *
+ * Update any windows just before placement
+ * so that placement algorithms will have the
+ * border size at place-time
+ */
+bool
+DecorWindow::place (CompPoint &pos)
+{
+    update (true);
+    return window->place (pos);
+}
+
 /*
  * DecorWindow::windowNotify
  *
@@ -2338,6 +2396,42 @@ DecorWindow::updateSwitcher ()
     isSwitcher = false;
 }
 
+DecorPixmapRequestorInterface *
+DecorScreen::findWindowRequestor (Window window)
+{
+    if (window == screen->root ())
+    {
+	return &mRequestor;
+    }
+    else
+    {
+	CompWindow *w = screen->findWindow (window);
+
+	if (w)
+	    return &(DecorWindow::get (w)->mRequestor);
+
+	return NULL;
+    }
+}
+
+DecorationListFindMatchingInterface *
+DecorScreen::findWindowDecorations (Window window)
+{
+    if (window == screen->root ())
+    {
+	return &decor[DECOR_ACTIVE];
+    }
+    else
+    {
+	CompWindow *w = screen->findWindow (window);
+
+	if (w)
+	    return &(DecorWindow::get (w)->decor);
+
+	return NULL;
+    }
+}
+
 
 /*
  * DecorScreen::handleEvent
@@ -2370,21 +2464,19 @@ DecorScreen::handleEvent (XEvent *event)
 	    if (event->xclient.message_type == requestFrameExtentsAtom)
 	    {
 		w = screen->findWindow (event->xclient.window);
-		if (w)
-		    DecorWindow::get (w)->update (true);
-	    }
-	    /* A decoration is pending creation, allow it to be created */
-	    if (event->xclient.message_type == decorPendingAtom)
-	    {
-		CompWindow *w = screen->findWindow (event->xclient.window);
-
 		if (w)
 		{
 		    DecorWindow *dw = DecorWindow::get (w);
 
-		    dw->mRequestor.handlePending (event->xclient.data.l);
+		    /* Set the frameExtentsRequested flag so that we know to
+		     * at least update _NET_WM_FRAME_EXTENTS (LP: #1110138) */
+		    dw->frameExtentsRequested = true;
+		    dw->update (true);
 		}
 	    }
+
+	    mCommunicator.handleClientMessage (event->xclient);
+	    break;
 	default:
 	    /* Check for damage events. If the output or input window
 	     * or a texture is updated then damage output extents.
@@ -2483,10 +2575,8 @@ DecorScreen::handleEvent (XEvent *event)
 		    }
 		    else
 		    {
-			int i;
-
 			/* A default decoration changed */
-			for (i = 0; i < DECOR_NUM; i++)
+			for (int i = 0; i < DECOR_NUM; i++)
 			{
 			    if (event->xproperty.atom == decorAtom[i])
 			    {
@@ -2551,54 +2641,24 @@ DecorScreen::handleEvent (XEvent *event)
 			if (dw->inputFrame ==
 			    ((XShapeEvent *) event)->window)
 			{
-			    XRectangle *shapeRects = 0;
-			    int order, n;
-
 			    dw->frameRegion = CompRegion ();
 
-			    shapeRects =
-				XShapeGetRectangles (screen->dpy (),
-				    dw->inputFrame, ShapeInput,
-				    &n, &order);
-			    if (!shapeRects || !n)
-				break;
-
-			    for (int i = 0; i < n; i++)
-				dw->frameRegion +=
-				    CompRegion (shapeRects[i].x,
-					        shapeRects[i].y,
-						shapeRects[i].width,
-						shapeRects[i].height);
+			    updateRegionWithShapeRectangles (screen->dpy (),
+							     dw->inputFrame,
+							     dw->frameRegion);
 
 			    w->updateFrameRegion ();
-
-			    XFree (shapeRects);
 			}
 			else if (dw->outputFrame ==
 			         ((XShapeEvent *) event)->window)
 			{
-			    XRectangle *shapeRects = 0;
-			    int order, n;
-
 			    dw->frameRegion = CompRegion ();
 
-			    shapeRects =
-				XShapeGetRectangles (screen->dpy (),
-				    dw->outputFrame, ShapeBounding,
-				    &n, &order);
-			    if (!n || !shapeRects)
-				break;
-
-			    for (int i = 0; i < n; i++)
-				dw->frameRegion +=
-				    CompRegion (shapeRects[i].x,
-					        shapeRects[i].y,
-						shapeRects[i].width,
-						shapeRects[i].height);
+			    updateRegionWithShapeRectangles (screen->dpy (),
+							     dw->outputFrame,
+							     dw->frameRegion);
 
 			    w->updateFrameRegion ();
-
-			    XFree (shapeRects);
 			}
 		    }
 		}
@@ -2784,9 +2844,7 @@ DecorWindow::moveNotify (int dx, int dy, bool immediate)
 {
     if (wd)
     {
-	int		 i;
-
-	for (i = 0; i < wd->nQuad; i++)
+	for (int i = 0; i < wd->nQuad; i++)
 	{
 	    wd->quad[i].box.x1 += dx;
 	    wd->quad[i].box.y1 += dy;
@@ -2823,13 +2881,7 @@ DecorWindow::resizeNotify (int dx, int dy, int dwidth, int dheight)
 	shading = false;
 	unshading = false;
     }
-    /* FIXME: we should not need a timer for calling decorWindowUpdate,
-       and only call updateWindowDecorationScale if decorWindowUpdate
-       returns false. Unfortunately, decorWindowUpdate may call
-       updateWindowOutputExtents, which may call WindowResizeNotify. As
-       we never should call a wrapped function that's currently
-       processed, we need the timer for the moment. updateWindowOutputExtents
-       should be fixed so that it does not emit a resize notification. */
+
     updateMatrix = true;
     updateReg = true;
 
@@ -2854,33 +2906,7 @@ DecorWindow::resizeNotify (int dx, int dy, int dwidth, int dheight)
 void
 DecorWindow::stateChangeNotify (unsigned int lastState)
 {
-    if (wd && wd->decor)
-    {
-	CompPoint oldShift = compiz::window::extents::shift (window->border (), window->sizeHints ().win_gravity);
-	
-
-	if ((window->state () & MAXIMIZE_STATE))
-	    window->setWindowFrameExtents (&wd->decor->maxBorder,
-					   &wd->decor->maxInput);
-	else
-	    window->setWindowFrameExtents (&wd->decor->border,
-					   &wd->decor->input);
-
-	/* Since we immediately update the frame extents, we must
-	 * also update the stored saved window geometry in order
-	 * to prevent the window from shifting back too far once
-	 * unmaximized */
-
-	CompPoint movement = compiz::window::extents::shift (window->border (), window->sizeHints ().win_gravity) - oldShift;
-
-	if (window->saveMask () & CWX)
-	    window->saveWc ().x += movement.x ();
-
-	if (window->saveMask () & CWY)
-	    window->saveWc ().y += movement.y ();
-
-	updateFrame ();
-    }
+    update (true);
 
     window->stateChangeNotify (lastState);
 }
@@ -3005,12 +3031,36 @@ DecorScreen::DecorScreen (CompScreen *s) :
 				   0,
 				   0,
 				   None,
-				   boost::shared_array <decor_quad_t> (NULL),
+				   boost::shared_array <decor_quad_t> (static_cast <decor_quad_t *> (NULL)),
 				   0,
 				   screen->root (),
 				   NULL)),
     mMenusClipGroup (CompMatch ("type=Dock | type=DropdownMenu | type=PopupMenu")),
-    mRequestor (screen->dpy (), screen->root (), &(decor[DECOR_ACTIVE]))
+    mRequestor (screen->dpy (), screen->root (), &(decor[DECOR_ACTIVE])),
+    mReleasePool (new PixmapReleasePool (
+		      boost::bind (XFreePixmap,
+				   screen->dpy (),
+				   _1))),
+    mPendingHandler (boost::bind (&DecorScreen::findWindowRequestor,
+				  this,
+				  _1)),
+    mUnusedHandler (boost::bind (&DecorScreen::findWindowDecorations,
+				 this,
+				 _1),
+		    mReleasePool,
+		    boost::bind (XFreePixmap,
+				 screen->dpy (),
+				 _1)),
+    mCommunicator (XInternAtom (screen->dpy (), DECOR_PIXMAP_PENDING_ATOM_NAME, FALSE),
+		   XInternAtom (screen->dpy (), DECOR_DELETE_PIXMAP_ATOM_NAME, FALSE),
+		   boost::bind (&PendingHandler::handleMessage,
+				&mPendingHandler,
+				_1,
+				_2),
+		   boost::bind (&UnusedHandler::handleMessage,
+				&mUnusedHandler,
+				_1,
+				_2))
 {
     supportingDmCheckAtom =
 	XInternAtom (s->dpy (), DECOR_SUPPORTING_DM_CHECK_ATOM_NAME, 0);
@@ -3033,9 +3083,9 @@ DecorScreen::DecorScreen (CompScreen *s) :
     decorSwitchWindowAtom =
 	XInternAtom (s->dpy (), DECOR_SWITCH_WINDOW_ATOM_NAME, 0);
     decorPendingAtom =
-	XInternAtom (s->dpy (), "_COMPIZ_DECOR_PENDING", 0);
+	XInternAtom (s->dpy (), DECOR_PIXMAP_PENDING_ATOM_NAME, 0);
     decorRequestAtom =
-	XInternAtom (s->dpy (), "_COMPIZ_DECOR_REQUEST", 0);
+	XInternAtom (s->dpy (), DECOR_REQUEST_PIXMAP_ATOM_NAME, 0);
     requestFrameExtentsAtom =
         XInternAtom (s->dpy (), "_NET_REQUEST_FRAME_EXTENTS", 0);
     shadowColorAtom =
@@ -3093,7 +3143,8 @@ DecorWindow::DecorWindow (CompWindow *w) :
     mClipGroup (NULL),
     mOutputRegion (window->outputRect ()),
     mInputRegion (window->inputRect ()),
-    mRequestor (screen->dpy (), w->id (), &decor)
+    mRequestor (screen->dpy (), w->id (), &decor),
+    lastMaximizedStateDecorated (0)
 {
     WindowInterface::setHandler (window);
 
@@ -3151,9 +3202,8 @@ DecorWindow::~DecorWindow ()
 bool
 DecorPluginVTable::init ()
 {
-    if (!CompPlugin::checkPluginABI ("core", CORE_ABIVERSION))
-	 return false;
+    if (CompPlugin::checkPluginABI ("core", CORE_ABIVERSION))
+	return true;
 
-    return true;
+    return false;
 }
-
