@@ -24,6 +24,8 @@
 
 COMPIZ_PLUGIN_20090315 (place, PlacePluginVTable)
 
+namespace cp = compiz::place;
+
 #define XWINDOWCHANGES_INIT {0, 0, 0, 0, 0, None, 0}
 
 PlaceScreen::PlaceScreen (CompScreen *screen) :
@@ -200,7 +202,9 @@ void
 PlaceScreen::handleEvent (XEvent *event)
 {
     if (event->type == ConfigureNotify &&
-	event->xconfigure.window == screen->root ())
+	event->xconfigure.window == screen->root () &&
+	(event->xconfigure.width != screen->width () ||
+	 event->xconfigure.height != screen->height ()))
     {
 	mPrevSize.setWidth (screen->width ());
 	mPrevSize.setHeight (screen->height ());
@@ -300,6 +304,14 @@ PlaceWindow::PlaceWindow (CompWindow *w) :
 
 PlaceWindow::~PlaceWindow ()
 {
+    if (!ps->mStrutWindows.empty() && window->struts())
+    {
+	ps->mStrutWindows.remove(window);
+	if (ps->mStrutWindows.empty())
+	{
+	    ps->doHandleScreenSizeChange(screen->width(), screen->height());
+	}
+    }
 }
 
 bool
@@ -338,46 +350,22 @@ PlaceWindow::place (CompPoint &pos)
 CompRect
 PlaceWindow::doValidateResizeRequest (unsigned int &mask,
 				      XWindowChanges *xwc,
-				      bool	   sizeOnly,
+				      bool	   onlyValidateSize,
 				      bool	   clampToViewport)
 {
-    CompRect workArea;
-    int	     x, y, left, right, bottom, top;
-    CompWindow::Geometry geom;
-    int      output;
-
-    geom.set (xwc->x, xwc->y, xwc->width, xwc->height,
-	      window->serverGeometry ().border ());
+    CompWindow::Geometry geom (xwc->x, xwc->y, xwc->width, xwc->height,
+			       window->serverGeometry ().border ());
+    CompPoint pos (geom.pos ());
 
     if (clampToViewport)
-    {
-	/* left, right, top, bottom target coordinates, clamed to viewport
-	 * sizes as we don't need to validate movements to other viewports;
-	 * we are only interested in inner-viewport movements */
+	pos = cp::getViewportRelativeCoordinates(geom, *screen);
 
-	x = geom.x () % screen->width ();
-	if ((geom.x2 ()) < 0)
-	    x += screen->width ();
+    CompWindowExtents edgePositions = cp::getWindowEdgePositions (pos,
+								  geom,
+								  window->border ());
 
-	y = geom.y () % screen->height ();
-	if ((geom.y2 ()) < 0)
-	    y += screen->height ();
-    }
-    else
-    {
-	x = geom.x ();
-	y = geom.y ();
-    }
-
-    left   = x - window->border ().left;
-    right  = left + geom.widthIncBorders () +  (window->border ().left +
-						window->border ().right);
-    top    = y - window->border ().top;
-    bottom = top + geom.heightIncBorders () + (window->border ().top +
-					       window->border ().bottom);
-
-    output   = screen->outputDeviceForGeometry (geom);
-    workArea = screen->getWorkareaForOutput (output);
+    int      output   = screen->outputDeviceForGeometry (geom);
+    CompRect workArea = screen->getWorkareaForOutput (output);
 
     if (clampToViewport &&
     	xwc->width >= workArea.width () &&
@@ -391,82 +379,40 @@ PlaceWindow::doValidateResizeRequest (unsigned int &mask,
 	}
     }
 
-    if ((right - left) > workArea.width ())
-    {
-	left  = workArea.left ();
-	right = workArea.right ();
-    }
-    else
-    {
-	if (left < workArea.left ())
-	{
-	    right += workArea.left () - left;
-	    left  = workArea.left ();
-	}
-
-	if (right > workArea.right ())
-	{
-	    left -= right - workArea.right ();
-	    right = workArea.right ();
-	}
-    }
-
-    if ((bottom - top) > workArea.height ())
-    {
-	top    = workArea.top ();
-	bottom = workArea.bottom ();
-    }
-    else
-    {
-	if (top < workArea.top ())
-	{
-	    bottom += workArea.top () - top;
-	    top    = workArea.top ();
-	}
-
-	if (bottom > workArea.bottom ())
-	{
-	    top   -= bottom - workArea.bottom ();
-	    bottom = workArea.bottom ();
-	}
-    }
+    cp::clampHorizontalEdgePositionsToWorkArea (edgePositions, workArea);
+    cp::clampVerticalEdgePositionsToWorkArea (edgePositions, workArea);
 
     /* bring left/right/top/bottom to actual window coordinates */
-    left   += window->border ().left;
-    right  -= window->border ().right + 2 * window->serverGeometry ().border ();
-    top    += window->border ().top;
-    bottom -= window->border ().bottom + 2 * window->serverGeometry ().border ();
+    cp::subtractBordersFromEdgePositions (edgePositions,
+					  window->border (),
+					  geom.border ());
 
     /* always validate position if the application changed only its size,
      * as it might become partially offscreen because of that */
-    if (!(mask) & (CWX | CWY) && (mask & (CWWidth | CWHeight)))
-	sizeOnly = false;
+    if (cp::onlySizeChanged (mask))
+	onlyValidateSize = false;
 
-    if ((right - left) != xwc->width)
-    {
-	xwc->width = right - left;
-	mask       |= CWWidth;
-	sizeOnly   = false;
-    }
+    if (cp::applyWidthChange(edgePositions,
+			     *xwc,
+			     mask))
+	onlyValidateSize = false;
 
-    if ((bottom - top) != xwc->height)
-    {
-	xwc->height = bottom - top;
-	mask        |= CWHeight;
-	sizeOnly    = false;
-    }
+    if (cp::applyHeightChange(edgePositions,
+			      *xwc,
+			      mask))
+	onlyValidateSize = false;
 
-    if (!sizeOnly)
+    if (!onlyValidateSize)
     {
-	if (left != x)
+	if (edgePositions.left != pos.x ())
 	{
-	    xwc->x += left - x;
+	    xwc->x += edgePositions.left - pos.x ();
 	    mask   |= CWX;
 	}
 
-	if (top != y)
+	if (edgePositions.top != pos.y ())
 	{
-	    xwc->y += top - y;
+	    xwc->y += edgePositions.top - pos.y ();
 	    mask   |= CWY;
 	}
     }
@@ -488,7 +434,9 @@ PlaceWindow::validateResizeRequest (unsigned int   &mask,
     if (!mask)
 	return;
 
-    if (source == ClientTypePager)
+    /* Clamp all windows initially on placement */
+    if (window->placed () &&
+	source == ClientTypePager)
 	return;
 
     if (window->state () & CompWindowStateFullscreenMask)
@@ -498,14 +446,16 @@ PlaceWindow::validateResizeRequest (unsigned int   &mask,
 			     CompWindowTypeDesktopMask))
 	return;
 
-    /* do nothing if the window was already (at least partially) offscreen */
-    if (window->serverX () < 0                         ||
-	window->serverX () + window->serverWidth () > screen->width () ||
-	window->serverY () < 0                         ||
-	window->serverY () + window->serverHeight () > screen->height ())
-    {
+    /* do nothing if the window was already (at least partially) offscreen
+     * and already placed */
+    bool onscreen =
+	CompRect (0,
+		  0,
+		  screen->width (),
+		  screen->height ()).contains (window->geometry ());
+
+    if (window->placed () && !onscreen)
 	return;
-    }
 
     if (hasUserDefinedPosition (false))
 	/* try to keep the window position intact for USPosition -
@@ -1195,36 +1145,10 @@ void
 PlaceWindow::constrainToWorkarea (const CompRect &workArea,
 				  CompPoint      &pos)
 {
-    CompWindowExtents extents;
-    int               delta;
-
-    extents.left   = pos.x () - window->border ().left;
-    extents.top    = pos.y () - window->border ().top;
-    extents.right  = extents.left + window->serverGeometry ().widthIncBorders () +
-		     (window->border ().left +
-		      window->border ().right);
-    extents.bottom = extents.top + window->serverGeometry ().heightIncBorders () +
-		     (window->border ().top +
-		      window->border ().bottom);
-
-    delta = workArea.right () - extents.right;
-    if (delta < 0)
-	extents.left += delta;
-
-    delta = workArea.left () - extents.left;
-    if (delta > 0)
-	extents.left += delta;
-
-    delta = workArea.bottom () - extents.bottom;
-    if (delta < 0)
-	extents.top += delta;
-
-    delta = workArea.top () - extents.top;
-    if (delta > 0)
-	extents.top += delta;
-
-    pos.setX (extents.left + window->border ().left);
-    pos.setY (extents.top  + window->border ().top);
+    pos = cp::constrainPositionToWorkArea (pos,
+                                           window->serverGeometry (),
+                                           window->border (),
+                                           workArea);
 
 }
 
@@ -1373,11 +1297,8 @@ PlaceWindow::grabNotify (int x,
 bool
 PlacePluginVTable::init ()
 {
-    if (!CompPlugin::checkPluginABI ("core", CORE_ABIVERSION))
-	return false;
+    if (CompPlugin::checkPluginABI ("core", CORE_ABIVERSION))
+	return true;
 
-    return true;
+    return false;
 }
-
-
-
