@@ -534,7 +534,6 @@ ShelfScreen::handleMotionEvent (unsigned int x,
     dy = y - lastPointerY;
 
     w->move (dx, dy, true);
-    w->syncPosition ();
 
     lastPointerX += dx;
     lastPointerY += dy;
@@ -817,13 +816,6 @@ ShelfWindow::~ShelfWindow ()
 bool
 ShelfPluginVTable::init ()
 {
-    if (!CompPlugin::checkPluginABI ("core", CORE_ABIVERSION))
-	return false;
-    if (!CompPlugin::checkPluginABI ("composite", COMPIZ_COMPOSITE_ABI))
-	return false;
-    if (!CompPlugin::checkPluginABI ("opengl", COMPIZ_OPENGL_ABI))
-	return false;
-
     if (!screen->XShape ())
     {
 	compLogMessage ("shelf", CompLogLevelError,
@@ -831,5 +823,10 @@ ShelfPluginVTable::init ()
 	return false;
     }
 
-    return true;
+    if (CompPlugin::checkPluginABI ("core", CORE_ABIVERSION)		&&
+	CompPlugin::checkPluginABI ("composite", COMPIZ_COMPOSITE_ABI)	&&
+	CompPlugin::checkPluginABI ("opengl", COMPIZ_OPENGL_ABI))
+	return true;
+
+    return false;
 }
